#include "HexSpinBoxDelegate.h"

#include <QSpinBox>
#include <QDebug>
#include <limits>

HexSpinBoxDelegate::HexSpinBoxDelegate(QObject *parent)
    : QItemDelegate(parent)
{
}

QWidget *HexSpinBoxDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    auto sb = new QSpinBox(parent);
    sb->setPrefix("0x");
    sb->setDisplayIntegerBase(16);
    sb->setMaximum(std::numeric_limits<int>::max());
//    sb->setSizeAdjustPolicy(QComboBox::AdjustToContents);
    sb->setMinimumWidth(150);

    auto val = index.model()->data(index, Qt::EditRole).toULongLong();
    sb->setValue(val);
    connect(sb, QOverload<int>::of(&QSpinBox::valueChanged),
            this, &HexSpinBoxDelegate::dataChanged);

    return sb;
}

void HexSpinBoxDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const
{
    auto value = index.model()->data(index, Qt::EditRole).toULongLong();
    auto sb = static_cast<QSpinBox*>(editor);
    if(sb->value() == value)
        return;
    QSignalBlocker block(sb);
    sb->setValue(value);
}

void HexSpinBoxDelegate::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
{
    auto cb = static_cast<QSpinBox*>(editor);
    auto val = cb->value();
    if(model->data(index, Qt::EditRole).toUInt() != val)
        model->setData(index, val, Qt::EditRole);
}

void HexSpinBoxDelegate::dataChanged(int val)
{
    commitData(qobject_cast<QWidget*>(sender()));
}
