#ifndef REGOPMODEL_H
#define REGOPMODEL_H

#include <QAbstractTableModel>
#include <QModelIndex>
#include <QVariant>

#include <optional>

#include "RegOpColumns.h"
#include "DreConfig.h"

class RegOpModel : public QAbstractTableModel
{
    Q_OBJECT
public:
    RegOpModel(QObject *parent = nullptr);
    QVariant data(const QModelIndex &index, int role) const override;
    virtual bool setData(const QModelIndex &index,
                         const QVariant &value,
                         int role = Qt::EditRole) override;
    Qt::ItemFlags flags(const QModelIndex &index) const override;
    QVariant headerData(int section, Qt::Orientation orientation,
                        int role) const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    int columnCount(const QModelIndex &parent = QModelIndex()) const override
    {
        Q_UNUSED(parent)
        return static_cast<int>(RegOpColumns::COL_TOTAL);
    }
    void setEnabled(bool en);

signals:
    void configChanged(const DreConfig &config) const;

public slots:
    void setConfig(const DreConfig &newConfig);
    void setRegResults(const QVector<quint64> &newResults);

private:
    QVariant backgroundData(const QModelIndex &index) const;
    QVariant checkStateData(const QModelIndex &index) const;
    QVariant displayData(const QModelIndex &index) const;
    QVariant editData(const QModelIndex &index) const;
    QVariant fontData(const QModelIndex &index) const;
    QVariant textAlignmentData(const QModelIndex &index) const;
    std::optional<RegOpAction> getRegOpAction(int i) const;
    std::optional<quint64> getRegResult(int i) const;

    DreConfig config;
    QVector<quint64> actionResults;
    bool enable = true;
};

#endif // REGOPMODEL_H
