#include "TriggerSetupDialog.h"
#include "ui_TriggerSetupDialog.h"

#include "config/HrbConfigDispatcher.h"
#include "device-modules/Hrb/HrbTriggerSetup.h"

TriggerSetupDialog::TriggerSetupDialog(QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::TriggerSetupDialog)
{
    ui->setupUi(this);

    map[ui->checkBoxTtlSpill] = &HrbTriggerSetup::ttlSpill;
    map[ui->checkBoxTtlTrigger] = &HrbTriggerSetup::ttlTrigger;
    map[ui->checkBoxLvdsSpill] = &HrbTriggerSetup::lvdsSpill;
    map[ui->checkBoxLvdsTrigger] = &HrbTriggerSetup::lvdsTrigger;
    map[ui->checkBoxInternalSpill] = &HrbTriggerSetup::internalSpill;
    map[ui->checkBoxInternalTrigger] = &HrbTriggerSetup::internalTrigger;
    map[ui->checkBoxSoftSpill] = &HrbTriggerSetup::softSpill;

    for(auto cb : map.keys())
        connect(cb, &QCheckBox::toggled, this, &TriggerSetupDialog::sendSetup);
    connect(ui->spinBoxInterval, QOverload<int>::of(&QSpinBox::valueChanged),
            this, &TriggerSetupDialog::sendSetup);

    HrbConfigDispatcher::instance().subscribeHrbTriggerSetup(this);
}

TriggerSetupDialog::~TriggerSetupDialog()
{
    delete ui;
}

void TriggerSetupDialog::setControlEnable(bool en)
{
    for(auto cb : map.keys())
        cb->setEnabled(en);
    ui->spinBoxInterval->setEnabled(en);
}

HrbTriggerSetup TriggerSetupDialog::getSetup() const
{
    HrbTriggerSetup setup;
    for(auto it=map.cbegin(); it!=map.cend(); ++it) {
        auto ref = it.value();
        setup.*ref = it.key()->checkState() == Qt::Checked;
    }
    setup.interval = ui->spinBoxInterval->value()/8;
    return setup;
}

void TriggerSetupDialog::setTriggerSetup(const ConfigKey &, const HrbTriggerSetup &setup)
{
    ++blockControl;
    for(auto it=map.cbegin(); it!=map.cend(); ++it) {
        auto ref = it.value();
        it.key()->setChecked(setup.*ref);
    }
    ui->spinBoxInterval->setValue(setup.interval*8);
    --blockControl;
}


void TriggerSetupDialog::on_spinBoxInterval_valueChanged(int interval)
{
    double rate = 1e9/interval;
    if(rate<1e4)
        ui->rateLabel->setText(QString("Up to: %1 Hz").arg(rate, 0, 'f', 0));
    else if(rate<1e7)
        ui->rateLabel->setText(QString("Up to: %1 kHz").arg(rate/1e3, 0, 'f', 1));
    else
        ui->rateLabel->setText(QString("Up to: %1 MHz").arg(rate/1e6, 0, 'f', 1));
}

void TriggerSetupDialog::sendSetup() const
{
    if(blockControl)
        return;
    emit hrbTriggerSetupChanged(getSetup());
}
