#ifndef HRBSTATUS_H
#define HRBSTATUS_H


#include <QMetaType>
#include <QMap>

#include "ModularDeviceStatus.h"
#include "base/DeviceIndex.h"
#include "base/fw_version.h"

struct HrbStatistic
{
    HrbStatistic & operator-=(const HrbStatistic& st);
    friend bool operator==(const HrbStatistic& l, const HrbStatistic& r) = default;

    quint32 trigPass = 0; // Reg 0x202, 32bit trigger pass counter, RO
    quint32 trigBlock = 0; // Reg 0x204, 32bit trigger block counter, RO
    quint32 tsZeroMismatch = 0; // Reg 0x300, 32bit trigger block counter, RO
    quint32 ttcTrigSyncLoss = 0; // Reg 0x302
    quint32 ttcTrigCodeError = 0; // Reg 0x304
    quint32 ttcResetSyncLoss = 0; // Reg 0x306
    quint32 ttcResetCodeError = 0; // Reg 0x308
    quint32 busyTimeoutCnt = 0; // Reg 0x20A
    quint64 busyTimer = 0; // Reg 0x210
    quint64 idleTimer = 0; // Reg 0x214
    quint64 busyMStreamTimer = 0; // Reg 0x220
    quint64 idleMStreamTimer = 0; // Reg 0x224
};

struct HrbStatus : public ModularDeviceStatus
{
    enum RTTC_MASK {
        RTTC_MASK_TTL_INPUT_ST = 0xF,
        RTTC_MASK_SPILL = 1<<4,
        RTTC_MASK_TRIG = 1<<5,
        RTTC_MASK_BUSY = 1<<6,
        RTTC_MASK_TTC_CLK = 1<<8,
        RTTC_MASK_TRIG_SYNC = 1<<9,
        RTTC_MASK_RESET_SYNC = 1<<10,
    };

    explicit HrbStatus(const DeviceIndex &deviceIndex = {})
        : ModularDeviceStatus(deviceIndex) {}

    HrbStatus & operator-=(const HrbStatus& st);
    friend bool operator==(const HrbStatus& l, const HrbStatus& r) = default;

    QString hostStr;
    QString fwStr; // Reg 0x56/0x57, FW_VER/FW_REV, 16-bit, firmware version/revision number, RO
    quint16 inState[8]; //* Reg 0x58 - 0x5F, Input State, RO
    quint16 rStatus = 0; // Reg 0x41, Status, R/O
    quint16 rttcStatus = 0; //* Reg 0x46, TTL/LVDS state
    quint32 evNum = 0; // Reg 0x206, 32bit spill event number, RO
    quint32 spillNum = 0; // Reg 0x208, 32bit spill number, RO
    quint64 globEvNum = 0; // Reg 0x218, 64bit event number, RO
    quint64 systemTs = 0; // Reg 0x20C, 64bit system timestamp, RO
    quint16 fifoSize = 0; // Reg 0x218, 16bit fifo size in 32-bit words , RO
    HrbStatistic statistic;
};
Q_DECLARE_METATYPE(HrbStatus)



////#include "device-modules/RunStatistic/RunStatistic.h"
////#include "device-modules/DeserializersControl/DeserializersControlStatus.h"
////#include "device-modules/TrigCtrlStat/TrigStatus.h"

//struct HrbStatistic
//{
//    void clear() {
//////        evNumAll=0;
//////        evNumCandidateRun=0;
//////        evNumAccepted=0;
////        timeValidLoss=0;
////        rxCodeError=0;
////        pllUnlockCnt=0;
////        runStatistic.clear();
//    }

//////    quint64 evNumAll;
//////    quint64 evNumCandidateRun;
//////    quint64 evNumAccepted;
////    quint32 timeValidLoss;
////    quint32 rxCodeError;
////    quint32 pllUnlockCnt;
////    RunStatistic runStatistic;
//};

//struct HrbStatus
//{
//    HrbStatus()
//    {
//        clear();
//    }
//    void clear() {
//        statistic.clear();
////        status=0;
////        desStatus.clear();
////        connected = false;
////        runFinished=true;
//    }

//    DeviceIndex index;
//    QString fwStr; // Reg 0x56/0x57, FW_VER/FW_REV, 16-bit, firmware version/revision number, RO
////    quint16 mcuVer = 0;
////    quint16 mcuRev = 0;
////    bool connected = false;
////    int tdcNCh=0;
////    AdcCoreStatus adcCoreStatus;
////    TrigStatus trigStatus;
////    bool runFinished = false;
////    QMap<int, double> temp;
////    quint16 status = 0; // 0x2 TM Time Valid
////    quint64 wrTime = 0;
////    DeserializersControlStatus desStatus;
////    quint16 pllStatus = 0;
////    bool mstreamMultiAck = false;
////    quint8 mstreamHwBufSize = 8;
////    QVector<double> hitRate;
////    QVector<HptdcStatus> hptdcStatus;

//    HrbStatistic statistic;
//};
//Q_DECLARE_METATYPE(HrbStatus)

#endif // HRBSTATUS_H
