/*
**    Copyright 2013-2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef TTBDEVICE_H
#define TTBDEVICE_H

#include <QMetaType>

#include "ModularMregDevice.h"
#include "TtbStatus.h"

class LegacyDeviceModule;
class UtcTimestampModule;

class TtbDevice : public ModularMregDevice
{
    Q_OBJECT
public:
    explicit TtbDevice(const DeviceIndex &index, QObject *parent = nullptr);

    virtual bool readTtbStatus();
    virtual bool readStatistics();
    virtual void clearStatistics();

signals:
    void statusUpdated(const TtbStatus &) const;
    void connectedStateChanged(const DeviceIndex &index, bool connected) const;
    void evNumResponse(int checkIndex, quint64 serial, quint64 evNum) const;

public slots:
    void softReset();
    void evNumRequest(int checkIndex);

protected slots:
    virtual void on_deviceConnected() override;
    virtual void on_deviceDisconnected() override;

protected:
    bool readStatus() override;
    bool writeConfig() override;

private:
    QString logPrefix() const { return getIdent(); }

    TtbStatus status;

    QSharedPointer<LegacyDeviceModule> legacyDevice;
    QSharedPointer<UtcTimestampModule> utcTimestampModule;
};

#endif // TTBDEVICE_H
