#ifndef MONGOVIEWERCONFIG_H
#define MONGOVIEWERCONFIG_H

#include <QDateTime>

#include "mongoqt/MongoCollection.h"
#include "mongoqt/MongoDatabase.h"
#include "mongoqt/MongoHost.h"

struct MongoViewerConfig
{
    enum Location {
        LOCAL,
        MONGO
    };
    enum Type {
        SAVED,
        HISTORY
    };
    friend bool operator==(const MongoViewerConfig &, const MongoViewerConfig &) = default;

    bool expand_tree = true;
    int rows_on_page = 20;
    int page_number = 1; // not save in mongo
    bool show_period = false; // not save in mongo
    QDateTime period_from = QDateTime::currentDateTime(); // not save in mongo
    QDateTime period_to = QDateTime::currentDateTime(); // not save in mongo
    bool show_run_number = false; // not save in mongo
    int run_number_from = 0; // not save in mongo
    int run_number_to = std::numeric_limits<int>::max(); // not save in mongo
    QString program_type_filter = ""; // not save in mongo
    QString program_index_filter = ""; // not save in mongo
    QString run_index_filter = ""; // not save in mongo
    bool show_all_collections = false; // not save in mongo
    MongoHost selected_host; // not save in mongo
    MongoDatabase selected_db; // not save in mongo
    MongoCollection selected_col; // not save in mongo
    Type type = SAVED;  // not save in mongo
    Location location = LOCAL;  // not save in mongo
};

Q_DECLARE_METATYPE(MongoViewerConfig)

#endif // MONGOVIEWERCONFIG_H
