#include "EventSizePeriod.h"

#include "params.h"

namespace {
constexpr const quint64 EV_SIZE_PERIOD_MIN_CNT = 1e3;
constexpr const qint64 EV_SIZE_PERIOD_MS = 1e4;
}

void EventSizePeriod::addEvent(quint32 s) {
    size += s;
    ++cnt;
}

void EventSizePeriod::newPeriod() {
    size = 0;
    cnt = 0;
    timeMark.start();
}

bool EventSizePeriod::isNextPeriod(const QElapsedTimer &curT) const {
    return cnt>EV_SIZE_PERIOD_MIN_CNT &&
            ( !timeMark.isValid() || (timeMark.msecsTo(curT)>EV_SIZE_PERIOD_MS));
}

std::optional<int> EventSizePeriod::getEvSize() const
{
    if(cnt)
        return size/cnt;
    else
        return {};
}

std::optional<int> EventSizePeriod::getNewBufSize() const
{
    if(!cnt)
        return {};

    int newEvSize = size/cnt;
    newEvSize = qMin(newEvSize, EVENT_SIZE_MAX);
    newEvSize = qMax(newEvSize, EVENT_SIZE_MIN);
    int newBufSize = newEvSize * EVENTS_IN_BUFER;
    return newBufSize;
}
