#include "MpdTluSignalRouter.h"

#include "MpdTluConfigDispatcher.h"
#include "MpdTLuCore.h"
#include "MpdTluDeviceGlobal.h"
#include "MpdTluDevice.h"
#include "MpdTluMainWindow.h"
#include "TluTtlIoWidget.h"
#include "Tlu16Sfp/Tlu16SfpModel.h"
#include "Tlu40Lvds/Tlu40LvdsModel.h"

#include "base/DeviceId.h"
#include "device-modules/ModularMregDevice.h"
#include "fsm/DeviceMgrFsm.h"
#include "lib-common/ReadoutAppConfigDb.h"
#include "lib-common/ReadoutAppCore.h"
#include "lib-common/ReadoutAppMainWindow.h"
#include "mldiscover/DeviceDispatcher.h"

MpdTluSignalRouter::MpdTluSignalRouter()
    : ReadoutAppSignalRouter()
{
    setDeviceIdFilter({DEVICE_ID_TLU16SFP, DEVICE_ID_TLU40LVDS});
    setReadoutAppConfigDispatcher(&MpdTluConfigDispatcher::instance());
}

void MpdTluSignalRouter::setCore(MpdTluCore *p)
{
    core = p;
    setReadoutAppCore(p);
}

void MpdTluSignalRouter::setMainWindow(MpdTluMainWindow *p)
{
    tluMw = p;
    tlu16SfpModel = p->getTlu16SfpModel();
    tlu40LvdsModel = p->getTlu40LvdsModel();
    tluTtlIoWidget16Sfp = p->getTluTtlIoWidget16Sfp();
    tluTtlIoWidget40Lvds = p->getTluTtlIoWidget40Lvds();
    setReadoutAppMainWindow(p);
}

void MpdTluSignalRouter::setReadoutAppConfigDispatcher(ReadoutAppConfigDispatcher *p)
{
    readoutConfDisp = p;
    setConfigDispatcher(p);
}

void MpdTluSignalRouter::connect_TluDeviceGlobal(const MpdTluDeviceGlobal *devGlob)
{
    if(!devGlob)
        return;

    auto dev = devGlob->getDevice<MpdTluDevice>();
    connect_device(devGlob, dev);

    // connect(core, &MpdTluCore::preRunReset,
    //         dev, &CruDevice::preRunReset);

    connect(core, &MpdTluCore::mstreamResetAll,
            devGlob, &MstreamMregDevGlobal::resetMStream);
    // if(cruFsm) {
    //     connect(dev, &CruDevice::preRunReseted,
    //             cruFsm, &CruFsm::setDevRunId);
    // }
    ;
    if (tlu16SfpModel) {
        connect(dev, &ModularMregDevice::tlu16SfpTrigBpUpdated,
                tlu16SfpModel, &Tlu16SfpModel::setTlu16SfpTrigBp);
        connect(dev, &ModularMregDevice::tluDecDesBpUpdated,
                tlu16SfpModel, &Tlu16SfpModel::setTluDesDecBp);

        connect(dev, &ModularMregDevice::tluDecDesStatusUpdated,
                tlu16SfpModel, &Tlu16SfpModel::setTluDecDesStatus);
        connect(dev, &ModularMregDevice::deviceConnected,
                tlu16SfpModel, &Tlu16SfpModel::onDeviceConnected);
    }
    if (tlu40LvdsModel) {
        connect(dev, &ModularMregDevice::tluDecDesBpUpdated,
                tlu40LvdsModel, &Tlu40LvdsModel::setTluDesDecBp);
        connect(dev, &ModularMregDevice::tlu40LvdsLuminosityBpUpdated,
                tlu40LvdsModel, &Tlu40LvdsModel::setTlu40LvdsLuminosityBp);
        connect(dev, &ModularMregDevice::tlu40LvdsTofTriggerBpUpdated,
                tlu40LvdsModel, &Tlu40LvdsModel::setTlu40LvdsTofTriggerBp);

        connect(dev, &ModularMregDevice::tlu40LvdsLuminosityStatusUpdated,
                tlu40LvdsModel, &Tlu40LvdsModel::setTlu40LvdsLuminosityStatus);
        connect(dev, &ModularMregDevice::tlu40LvdsTofTriggerStatusUpdated,
                tlu40LvdsModel, &Tlu40LvdsModel::setTlu40LvdsTofTriggerStatus);
        connect(dev, &ModularMregDevice::deviceConnected,
                tlu40LvdsModel, &Tlu40LvdsModel::onDeviceConnected);
    }

    for(auto w : {tluTtlIoWidget16Sfp, tluTtlIoWidget40Lvds}) {
        if(!w)
            continue;
        connect(dev, &MpdTluDevice::ttlIoStartupCfg,
                w, &TluTtlIoWidget::setTtlIoStartupCfg);
        if(configDispatcher)
            configDispatcher->subscribeConfig(w);
    }

    if(readoutAppMainWindow){
        connect(dev, &ModularMregDevice::statusUpdated,
                readoutAppMainWindow, &ReadoutAppMainWindow::setStatus);

        // connect(dev, &TqdcDevice::statusUpdated,
        //         mainWindow, &TqdcMainWindow::deviceStatusUpdated);
    }
// #ifdef QT_WIDGETS_LIB
//     if(mainWindow){
//         if (auto w = mainWindow->getTrigCtrlStatWidget()) {
//             connect(dev, &ModularMregDevice::trigCtrlBpUpdated,
//                     w, &TrigCtrlStatWidget::setTrigCtrlBp);
//         }
//     }
// #endif

    deviceDispatcher->subscribeHostAddressUpdates(devGlob->getDeviceIndex(),
                                                  dev);
}

void MpdTluSignalRouter::connect_ConfigurationDispatcher() const
{
    ReadoutAppSignalRouter::connect_ConfigurationDispatcher();

    Q_ASSERT(configDispatcher);
    if (deviceDispatcher)
        deviceDispatcher->subscribeDevices(configDispatcher); // TODO: move to ReadoutAppSignalRouter

    if(core) {
        configDispatcher->subscribeConfig(core);
    }

    if(configDb) {
        configDispatcher->subscribeConfig(configDb);
        configDispatcher->subscribeCS(configDb);
    }

    for(auto w : {tluTtlIoWidget16Sfp, tluTtlIoWidget40Lvds}) {
        if(!w)
            continue;
        configDispatcher->subscribeConfig(w);
    }

    if(tlu16SfpModel)
        configDispatcher->subscribeConfig(tlu16SfpModel);
    if(tlu40LvdsModel)
        configDispatcher->subscribeConfig(tlu40LvdsModel);
}

void MpdTluSignalRouter::connect_FSM() const
{
    ReadoutAppSignalRouter::connect_FSM();

    if (!deviceMgrFsm)
        return;
    if (remoteControlServer) {
    }

    if(tluMw) {
        connect(tluMw, &MpdTluMainWindow::printFsmInputs,
                deviceMgrFsm, &BaseFsm::printFsmInput);

        connect(deviceMgrFsm, &BaseFsm::stateChanged,
                tluMw, &MpdTluMainWindow::onFsmStateChange);
    }

    connect(deviceMgrFsm, &BaseFsm::stateChanged,
            tluTtlIoWidget16Sfp, &TluTtlIoWidget::onFsmStateChange);

    for(auto w : {tluTtlIoWidget16Sfp, tluTtlIoWidget40Lvds}) {
        if(!w)
            continue;
        connect(deviceMgrFsm, &BaseFsm::stateChanged,
                w, &TluTtlIoWidget::onFsmStateChange);
    }
}

void MpdTluSignalRouter::connect_MainWindow() const
{
    ReadoutAppSignalRouter::connect_MainWindow();

    if(!tluMw)
        return;

    if(tlu16SfpModel) {
        connect(tluMw, &MpdTluMainWindow::resetCntSfp,
                tlu16SfpModel, &Tlu16SfpModel::resetCounters);
    }

    if(tlu40LvdsModel) {
        connect(tluMw, &MpdTluMainWindow::resetCntLvds,
                tlu40LvdsModel, &Tlu40LvdsModel::resetCounters);
    }

    if(core) {
        connect(tluMw, &MpdTluMainWindow::performLocalStart,
                core, &MpdTluCore::performLocalStart);
        connect(tluMw, &MpdTluMainWindow::performLocalStop,
                core, &MpdTluCore::performLocalStop);
    }
}
