#include "Tlu40LvdsWidget.h"
#include "ui_Tlu40LvdsWidget.h"
#include "Tlu40LvdsModel.h"
#include "Tlu40LvdsColumns.h"

#include "base/Globals.h"
#include "ItemDelegate/CommonSpinBoxDelegate.h"

namespace {
static const int COL_SHAPER = static_cast<int>(Tlu40LvdsColumns::COL_SHAPER);
}

Tlu40LvdsWidget::Tlu40LvdsWidget(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::Tlu40LvdsWidget)
    , model(new Tlu40LvdsModel(this))
    , shaperSbDelegate(new CommonSpinBoxDelegate(this))
{
    ui->setupUi(this);
    ui->treeView->setModel(model);
    connect(model, &Tlu40LvdsModel::configChanged,
            this, &Tlu40LvdsWidget::configChanged);
    connect(model, &Tlu40LvdsModel::modelReset,
            this, &Tlu40LvdsWidget::onModelReset);

    ui->treeView->setItemDelegateForColumn(COL_SHAPER, shaperSbDelegate);

    ui->treeView->hideColumn(static_cast<int>(Tlu40LvdsColumns::COL_DEC_DES_RESET));
    ui->treeView->header()->setStretchLastSection(false);
    ui->treeView->header()->setSectionResizeMode(QHeaderView::ResizeToContents);
}

Tlu40LvdsWidget::~Tlu40LvdsWidget()
{
    delete ui;
}

void Tlu40LvdsWidget::updateRemoteBlocking(bool remoteConnected) const
{

}

void Tlu40LvdsWidget::setConfig(const ConfigKey &config_key, const BaseDeviceAppConfig &newConfig)
{
    // Q_UNUSED(config_key)
    if (appConfig == newConfig)
        return;

    appConfig = newConfig;
    model->setConfig(config_key, appConfig);
}

void Tlu40LvdsWidget::onFsmStateChange(const FsmState &state)
{
    bool en = state==FsmState::Ready || state==FsmState::Error
        || Globals::getDebugMode();
    setEnabled(en);
}

void Tlu40LvdsWidget::onModelReset()
{
    addDelegate({});
    ui->treeView->expandAll();
    updateGeometry();

    for(auto c : {COL_SHAPER}) {
        ui->treeView->resizeColumnToContents(c);
    }
}

void Tlu40LvdsWidget::addDelegate(const QModelIndex &index)
{
    qInfo() << __PRETTY_FUNCTION__ << model->data(index, Qt::DisplayRole);
    auto view = ui->treeView;
    for(auto c : {COL_SHAPER}) {
        auto sibIndex = index.siblingAtColumn(c);
        if(!sibIndex.isValid())
            continue;
        if(!view->isPersistentEditorOpen(sibIndex))
            view->openPersistentEditor(sibIndex);
    }

    int chRows = model->rowCount(index);
    for(int i=0; i<chRows; ++i) {
        auto chIndex = model->index(i, 0, index);
        addDelegate(chIndex);
    }
}
