#include "MscHistWidget.h"

#include <QLayout>
#include <QTimer>
#include <QDebug>
#include <QMetaType>

#include "qxw/WaveformPlot.h"
#include "qwt_scale_engine.h"
#include "qwt_plot_zoomer.h"
#include "data/MscHistData.h"
#include "config/MscConfigDispatcher.h"

MscHistWidget::MscHistWidget(int hId, bool useLogScale, QWidget *parent)
    : WaveformPlot(parent)
    , hId(hId)
{
    auto &configDispatcher = MscConfigDispatcher::instance();
    connect(&configDispatcher, &BaseConfigDispatcher::deviceConfigChanged,
            this, &MscHistWidget::setDeviceConfig);

    setMinimumHeight(200);
    setMinimumWidth(200);
    if(useLogScale) {
        setAxisScaleEngine(QwtPlot::xBottom, new QwtLog10ScaleEngine());
        setAxisScaleEngine(QwtPlot::yLeft, new QwtLog10ScaleEngine());
    } else {
        setAxisScaleEngine(QwtPlot::xBottom, new QwtLinearScaleEngine());
        setAxisScaleEngine(QwtPlot::yLeft, new QwtLinearScaleEngine());
    }

    setTitleName(QString("%1").arg(this->hId));
    enableLegend(false);
}

void MscHistWidget::setXAxisTitle(QString title)
{
    QFont font;
    font.setWeight(0);
    QwtText title_axis_bottom(title);
    title_axis_bottom.setFont(font);
    setAxisTitle(QwtPlot::xBottom, title_axis_bottom);
}

void MscHistWidget::setYAxisTitle(QString title)
{
    QFont font;
    font.setWeight(0);
    QwtText title_axis_bottom(title);
    title_axis_bottom.setFont(font);
    setAxisTitle(QwtPlot::yLeft, title_axis_bottom);
}

void MscHistWidget::updateHistogram(const MscHistData &h)
{
    WaveformDataAll wda;
    WaveformData wd0;

    for(int i=0; i<h.size(); ++i){
        wd0.c = 4;
        wd0.p.push_back(point_t(h[i].x, h[i].y));
    }

    wda.push_back(wd0);

    clear();
    setAutoReplot(false);
    setAllData(wda);
}

void MscHistWidget::setDeviceConfig(const ConfigKey &key, const ModularDeviceConfig &newConfig)
{
    Q_UNUSED(key)
    if(config == newConfig)
        return;
    config = newConfig;

    setTitleName(getMscHistDisplayName(config.mscCore.hist, hId));
}

void MscHistWidget::setAScale(const MscHistData &h)
{
    if(h.size() < 1)
        return;

    double y_min=h[0].y;
    double y_max=h[0].y;
    for(int i=0; i<h.size(); i++){
        if(h[i].y > y_max)
            y_max = h[i].y;
        if(h[i].y < y_min)
            y_min = h[i].y;
    }
    setAxisScale(QwtPlot::yLeft, 0.1, y_max);
}

void MscHistWidget::setTitleName(QString name)
{
    QwtText title(name);
    QFont font;
    font.setWeight(0);
    title.setFont(font);
    setTitle(title);
}

