/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef ABSTRACTDEVICECONTROLLER_H
#define ABSTRACTDEVICECONTROLLER_H

#include <QObject>
#include <QSet>

#include "base/DeviceId.h"
#include "AbstractConfigurable.h"

class DeviceDispatcher;
class DeviceIndexEnabledMap;
struct RcConfig;
class RcConfigDispatcher;

class AbstractDeviceController : public AbstractConfigurable
{
    Q_OBJECT
public:
    explicit AbstractDeviceController(QObject *parent = nullptr);
    virtual ~AbstractDeviceController() = default;

    virtual void setDeviceDispatcher(DeviceDispatcher *p);

    virtual QSet<DeviceId> getDeiveIdFilter();

signals:
    void deviceListChanged(const DeviceIndexEnabledMap &); // may be unused

public slots:
    virtual void setDeviceList(const DeviceIndexEnabledMap &devices) = 0;

protected:
    DeviceDispatcher * deviceDispatcher = nullptr;
};

#endif // ABSTRACTDEVICECONTROLLER_H
