#ifndef CONFIGMETA_H
#define CONFIGMETA_H

#include <QMetaType>
#include <QDateTime>
#include <QHostInfo>

class ConfigMeta{
public:
    friend class ConfigMetaConverter;
    ConfigMeta() {}
    ConfigMeta(const QHostInfo &_current_host,
               const QDateTime &_date_of_modification,
               const bool _phantom)
        :
        current_host(_current_host),
        date_of_modification(_date_of_modification),
        phantom(_phantom)
    {}
    void updateModificationTimestamp() {
        date_of_modification = QDateTime::currentDateTime();
    }
    void setPhantom(const bool &ph){
        phantom = ph;
    }
    bool isPhantom() const { return phantom; }
    QHostInfo getCurrentHost() const { return current_host; }
    QDateTime getModificationTimestamp() const { return date_of_modification; }
    void setModificationTimestamp(const QDateTime &dt) { date_of_modification = dt; }
    void setCurrentHost(const QHostInfo &hi) { current_host = hi; }
    QString toString() const;
    friend bool operator==(const ConfigMeta &l, const ConfigMeta &r);

private:
    QHostInfo current_host = QHostInfo::fromName(QHostInfo::localHostName());
    QDateTime date_of_modification = QDateTime::currentDateTime();
    bool phantom = false;
};

Q_DECLARE_METATYPE(ConfigMeta)

#endif // CONFIGMETA_H
