/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "ProgramIndex.h"

#include <QDataStream>
#include <QDebug>
#include <QStringList>

QString ProgramIndex::toString() const
{
    if (isDomainDefault())
        return getIndex();
    return getDomain() + "." + getIndex();
}

ProgramIndex ProgramIndex::fromString(const QString &r)
{
    const auto list = r.split(".");
    if (list.size() < 2)
        return ProgramIndex(QString(), r);
    return ProgramIndex(list[0], list[1]);
}

QDataStream &operator<<(QDataStream &out, const ProgramIndex &data)
{
    out << data.domain << data.index;
    return out;
}

QDataStream &operator>>(QDataStream &in, ProgramIndex &data)
{
    QString domain, index;
    in >> domain >> index;
    if (in.status() == QDataStream::Ok) {
        data = ProgramIndex(domain, index);
    }
    return in;
}

QDebug operator<<(QDebug dbg, const ProgramIndex &r)
{
    dbg.noquote().nospace()
            << "ProgramIndex("
            << "domain: " << r.domain
            << ", index: " << r.index
            << ")";
    return dbg;
}
