#ifndef FPGA_MCU_REGS_ADDR
#define FPGA_MCU_REGS_ADDR

const int REG_BMC_COUNT = 0x20;

// MCU Registers v1.0
enum {
    REG_BMC_V1_VERSION            = 0x00,
    REG_BMC_V1_ONBOARD_PLL_STATUS = 0x01,
    REG_BMC_V1_PLL_UNLOCK_CNT     = 0x02, // [12:0] - unlock counter
    REG_BMC_V1_TEMP_1             = 0x03,
    REG_BMC_V1_TEMP_2             = 0x04,
    REG_BMC_V1_TEMP_3             = 0x05,
    REG_BMC_V1_TEMP_4             = 0x06,
    REG_BMC_V1_REVISION           = 0x07
};

// MCU Registers v2.0
enum {
    REG_BMC_DEV_ID             = 0x00,
    REG_BMC_ONBOARD_PLL_STATUS = 0x01,
    REG_BMC_PLL_UNLOCK_CNT     = 0x02, // [12:0] - unlock counter
    REG_BMC_TEMP_1             = 0x03, // pll temp
    REG_BMC_TEMP_2             = 0x04, // hptdc_group_a temp
    REG_BMC_TEMP_3             = 0x05, // hptdc_group_b temp
    REG_BMC_TEMP_4             = 0x06, // hptdc_group_c temp
    REG_BMC_VERSION            = 0x08,
    REG_BMC_REVISION           = 0x09,
    REG_BMC_SYSTEM_STATUS      = 0x0A, // [1:0] - 0: unknown, 1: normal, 2: warning, 3: critical
    REG_BMC_POWER_STATUS       = 0x0B,
    REG_BMC_PLL_STATUS         = 0x0C,

    REG_BMC_VOL_TDC_A          = 0x10, // mV
    REG_BMC_CUR_TDC_A          = 0x11, // mA
    REG_BMC_VOL_TDC_B          = 0x12,
    REG_BMC_CUR_TDC_B          = 0x13,
    REG_BMC_VOL_TDC_C          = 0x14,
    REG_BMC_CUR_TDC_C          = 0x15,

    REG_BMC_VOL_BP_5           = 0x10, // Backplane +5V supply voltage, mV
    REG_BMC_CUR_BP_5           = 0x11, // Backplane +5V supply current, mA
    REG_BMC_VOL_BP_33          = 0x12, // Backplane +3.3V supply voltage, mV
    REG_BMC_CUR_BP_33          = 0x13, // Backplane +3.3V supply current, mA

    REG_BMC_VOL_TDC_D          = 0x1A,
    REG_BMC_CUR_TDC_D          = 0x1B,
};

#endif // FPGA_MCU_REGS_ADDR
