#include "CalResultConverter.h"

#include "base/ConfigConverterUtil.h"

#include "CalResult.h"
// BaselineStats
// MultichannelBaselineStats
// CalResult


namespace {
// BaselineStats
constexpr const char *k_sampleCount = "sampleCount";
constexpr const char *k_min = "min";
constexpr const char *k_max = "max";
constexpr const char *k_mean = "mean";
constexpr const char *k_stddev = "stddev";
constexpr const char *k_median = "median";
constexpr const char *k_temps = "temps";

// CalResult
constexpr const char *k_index = "index";
constexpr const char *k_nch = "nch";

constexpr const char *k_baseline_x1 = "baseline_x1";
constexpr const char *k_baseline_x4 = "baseline_x4";
constexpr const char *k_dacCal_mV = "dacCal_mV";
}






template<>
bool AbstractConfigConverter<BaselineStats>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<BaselineStats>::toJsonObject(const BaselineStats &in)
{
    QVariantMap m;
    m[k_sampleCount] = QVariant::fromValue(in.sampleCount);
    m[k_min] = QVariant::fromValue(in.min);
    m[k_max] = QVariant::fromValue(in.max);
    m[k_mean] = QVariant::fromValue(in.mean);
    m[k_stddev] = QVariant::fromValue(in.stddev);
    m[k_median] = QVariant::fromValue(in.median);
    // m[k_] = QVariant::fromValue(in.program_type);
    return fromVariantMap(m);
}

template<>
BaselineStats AbstractConfigConverter<BaselineStats>::fromJsonObject(const QJsonObject &m)
{
    BaselineStats out;
    update_value(m, k_sampleCount, &out.sampleCount);
    update_value(m, k_min, &out.min);
    update_value(m, k_max, &out.max);
    update_value(m, k_mean, &out.mean);
    update_value(m, k_stddev, &out.stddev);
    update_value(m, k_median, &out.median);
    // update_value(m, k_, &out.max);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<BaselineStats>::toJsonValue(const BaselineStats &in)
{
    return toJsonObject(in);
}

template<>
BaselineStats AbstractConfigConverter<BaselineStats>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<BaselineStats>::registerSubTypes()
{}






template<>
bool AbstractConfigConverter<MultichannelBaselineStats>::classIsObject()
{
    return true;
}

template<>
QJsonValue AbstractConfigConverter<MultichannelBaselineStats>::toJsonValue(const MultichannelBaselineStats &in)
{
    return ConfigConverterUtil::toJsonValue(in);
}

template<>
QJsonObject AbstractConfigConverter<MultichannelBaselineStats>::toJsonObject(const MultichannelBaselineStats &in)
{
    return toJsonValue(in).toObject();
}

template<>
MultichannelBaselineStats AbstractConfigConverter<MultichannelBaselineStats>::fromJsonObject(const QJsonObject &m)
{
    const auto &map = ConfigConverterUtil::toMap<int, BaselineStats>(m);
    MultichannelBaselineStats out;
    for(auto it=map.cbegin(); it!=map.cend(); ++it)
        out[it.key()] = *it;
    return out;
}

template<>
MultichannelBaselineStats AbstractConfigConverter<MultichannelBaselineStats>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<MultichannelBaselineStats>::registerSubTypes()
{
    AbstractConfigConverter<BaselineStats>::registerTypes();
}

template<>
bool AbstractConfigConverter<CalResult>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<CalResult>::toJsonObject(const CalResult &in)
{
    QVariantMap m;
    m[k_index] = QVariant::fromValue(in.index);
    m[k_nch] = QVariant::fromValue(in.nch);
    m[k_temps] = ConfigConverterUtil::toJsonValue(in.temps);
    if(!in.baseline_x1.isEmpty())
        m[k_baseline_x1] = QVariant::fromValue(in.baseline_x1);
    m[k_baseline_x4] = QVariant::fromValue(in.baseline_x4);
    m[k_dacCal_mV] = ConfigConverterUtil::toJsonValue(in.dacCal_mV);
    return fromVariantMap(m);
}

template<>
CalResult AbstractConfigConverter<CalResult>::fromJsonObject(const QJsonObject &m)
{
    CalResult out;
    update_value(m, k_index, &out.index);
    update_value(m, k_nch, &out.nch);
    out.temps = ConfigConverterUtil::toMap<QString, double>(m[k_temps]);
    update_value(m, k_baseline_x1, &out.baseline_x1);
    update_value(m, k_baseline_x4, &out.baseline_x4);
    auto map = ConfigConverterUtil::toMap<int, double>(m[k_dacCal_mV]);
    out.dacCal_mV = map;
    return out;
}

template<>
QJsonValue AbstractConfigConverter<CalResult>::toJsonValue(const CalResult &in)
{
    return toJsonObject(in);
}

template<>
CalResult AbstractConfigConverter<CalResult>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<CalResult>::registerSubTypes()
{
    AbstractConfigConverter<DeviceIndex>::registerTypes();
    AbstractConfigConverter<MultichannelBaselineStats>::registerTypes();
}
