//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/*
#include <vector>

#include "dsplib/FirCreator.h"
#include "dsplib/Convolver.h"
#include "dsplib/Deconvolver.h"
#include "dsplib/VectorOp.h"
#include "dsplib/fir.h"
#include "util/TextFormatter.h"
#include "dsplib/LinearSweepMinimizer.h"

FirCreator::FirCreator(const std::vector<double> &sourceImpulse, const std::vector<double> &targetImpulse)
    : deconvolver(new Deconvolver), src(sourceImpulse), tgt(targetImpulse),
    maxAmplification(3.0),
    windowingEnabled(true), windowWidth(0.5), windowType(WF_BLACKMAN),
    refPulseWidth(1.0),
    refPulseTime(0.1),
    refPulseTimeStep(1.0)
{
}

FirCreator::~FirCreator()
{
    delete deconvolver;
}

typedef std::vector<double> FirCoef;
typedef std::vector<double> ImpulseData;

void FirCreator::runOnce()
{
    firCoef = deconvolver->make_deconvolution_dblFIR (src, tgt, 0, 0, maxAmplification);
    if (!windowingEnabled) return;
    firCoef = applyWindowing(firCoef, windowType, windowWidth);
}


//-----------------------------------------------------------------------------

//{
//public:
//    FirEvaluatorParSet();
//    double windowWidth;
//    double maxAmplification;
//};
//FirEvaluatorParSet::FirEvaluatorParSet()
//    : windowWidth(0.5), maxAmplification(5.0)
//{
//}

// evaluate fir quality using window width as parameter
class FirEvaluator {
public:
    FirEvaluator(const FirCoef &fc, const ImpulseData &src, const ImpulseData &tgt);
    virtual ~FirEvaluator();
//    void setFirCoef(const FirCoef &fc) { p_firCoef=fc; }
    double operator()(const MinimizerParSet *par) const;
    FirCoef getFirCoef(const MinimizerParSet &par) const;
protected:
    std::vector<double> getTargetImpulse(const MinimizerParSet &par) const;
    FirCoef applyWindowing(double windowWidth) const;

private:
    Deconvolver *deconvolver;
    const FirCoef p_firCoef;
    const ImpulseData &p_src, p_tgt;
};

FirEvaluator::FirEvaluator(const FirCoef &fc, const ImpulseData &src, const ImpulseData &tgt)
    : deconvolver(new Deconvolver), p_firCoef(fc), p_src(src), p_tgt(tgt)
{
}

FirEvaluator::~FirEvaluator()
{
    delete deconvolver;
}

FirCoef FirEvaluator::applyWindowing(double windowWidth) const
{
    return FirCreator::applyWindowing(p_firCoef, WF_BLACKMAN, windowWidth);
}

std::vector<double> FirEvaluator::getTargetImpulse(const MinimizerParSet &par) const
{
    size_t size = p_src.size();
    double timeStep = par.getValue("refPulseTimeStep");
    double pulseWidth = par.getValue("refPulseWidth");
    double pulseTime = par.getValue("refPulseTime");
//    std::cerr << "getTargetImpulse: pulseTime=" << pulseTime
//            << ", pulseWidth="<< pulseWidth
//            << ", timeStep=" << timeStep
//            << std::endl;
    std::vector<double> out(size);
    for(size_t i=0; i<out.size(); i++) {
        const double t = double(i)*timeStep - pulseTime;
        out[i] = gauss(t, pulseWidth/GausFWHM);
    }
    return out;
}

FirCoef FirEvaluator::getFirCoef(const MinimizerParSet &par) const
{
    const std::vector<double> &tgt = getTargetImpulse(par);
    FirCoef firCoef = deconvolver->make_deconvolution_dblFIR (p_src, tgt, 0, 0, par.getValue("maxAmplification"));
    const FirCoef &curFirCoef = FirCreator::applyWindowing(firCoef, WF_BLACKMAN, par.getValue("windowWidth"));
    return curFirCoef;
}

double FirEvaluator::operator()(const MinimizerParSet *par) const
{
    const std::vector<double> &tgt = getTargetImpulse(*par);
    const FirCoef &curFirCoef = getFirCoef(*par);
    //    const FirCoef &curFirCoef = applyWindowing(par.windowWidth);
    const size_t outlen = p_src.size()+curFirCoef.size()-1;
    std::vector<double> out(outlen);
    out = Convolver::convolve(curFirCoef, p_src);
    std::vector<double> diff = out - tgt;
    return enorm(diff);
}

void FirCreator::optimize()
{
    firCoef = deconvolver->make_deconvolution_dblFIR (src, tgt, 0, 0, maxAmplification);
    if (!windowingEnabled) return;

    FirEvaluator firEvaluator(firCoef, src, tgt);

//    const FirCoef &curFirCoef = firEvaluator.applyWindowing(windowWidth);
//    std::cerr << "start: " << firEvaluator(windowWidth) << std::endl;


    // find appropriate windowing parameters
    // optimization function has multiple local minima, scan over all range 0.1-1.0
    {
        MinimizerParSet ps;
        ps.insert(MinimizerPar("windowWidth", windowWidth, 0.1, 2.0));
        ps.insert(MinimizerPar("maxAmplification", 5.0));
        ps.insert(MinimizerPar("refPulseTimeStep", refPulseTimeStep));
        ps.insert(MinimizerPar("refPulseTime", refPulseTime, 0.0, 2.0));
        ps.insert(MinimizerPar("refPulseWidth", refPulseWidth, 0.1, 2.0));
        MinStepper<FirEvaluator> minStepper(firEvaluator, ps);
        StepParam sp(ps);
//        sp = minStepper.run(100, "windowWidth");
        sp = minStepper.run(100, "refPulseTime");
        sp = minStepper.run(100, "refPulseWidth");
        sp = minStepper.run(100, "refPulseTime");
//        ps = sp.par;
        setWindowWidth(sp.par.getValue("windowWidth"));
        setRefPulseTime(sp.par.getValue("refPulseTime"));
        setRefPulseWidth(sp.par.getValue("refPulseWidth"));
        //firCoef = firEvaluator.applyWindowing(windowWidth);
        firCoef = firEvaluator.getFirCoef(sp.par);
    }
}

std::vector<double> FirCreator::applyWindowing(const std::vector<double> &v, fwinf_t p_windowType, double p_windowWidth)
{
    std::vector<double>  out(v.size());
    for(size_t i=0; i<v.size(); i++) {
        const size_t w = v.size()*(p_windowWidth);
        out[i] = v[i] * windowingFunction(p_windowType, i+double(w), 2.0*w);
    }
    return out;
}

std::vector<double> FirCreator::runFir(const std::vector<double> &firCoef) const
{
    // convolve
    const size_t outlen = src.size()+firCoef.size()-1;
    std::vector<double> out(outlen);
    out = Convolver::convolve(firCoef, src);
    return out;
}

double FirCreator::getFirError(const std::vector<double> &firCoef) const
{
    std::vector<double> out = runFir(firCoef);
    std::vector<double> diff = out - tgt;
    double norm = enorm(diff);
    return norm;
}
*/
