#ifndef BASEFSM_H
#define BASEFSM_H

#include <QElapsedTimer>
#include <QObject>

#include "FsmState.h"
#include <QSharedData>

struct BaseFsmInput : public QSharedData
{
    void clearRequests() {
        requestState = FsmState::Init;
        requestTime.invalidate();
    }

    FsmState requestState = FsmState::Init;
    QElapsedTimer requestTime;
};

class BaseFsm: public QObject
{
    Q_OBJECT

public:
    explicit BaseFsm(QObject *parent = nullptr);
    FsmState getState() const { return state; }
    virtual bool isTransitionAllowed(const FsmState &toState) const;

public slots:
    bool setRequestState(const FsmState &nextState);
    virtual void printFsmInput() const;

signals:
    void stateChanged(const FsmState &cur, const FsmState &old) const;

protected slots:
    virtual bool evaluate();

protected:
    virtual void printInputString(QDebug dbg) const;
    FsmState getRequestState() const { return input->requestState; }
    bool hasRequestState() const { return input->requestState != FsmState::Init; }
    virtual qint64 stateTimeoutMs(const FsmState &state) const;
    virtual qint64 requestTimeoutMs() const;
    void setState(const FsmState &nextState);
    void acceptRequestedState();
    BaseFsmInput* getInput() { return input; }
    const BaseFsmInput* getInput() const { return input; }
    void recreateInput(BaseFsmInput* p);

private:
    bool stateTimeout() const;
    bool requestTimeout() const;

    QElapsedTimer stateTime;
    FsmState state = FsmState::Init;
    QSharedDataPointer<BaseFsmInput> input;
};

QDebug operator<<(QDebug dbg, const BaseFsmInput &state);

#endif // BASEFSM_H
