/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef DEVICEENABLETABLEMODEL_H
#define DEVICEENABLETABLEMODEL_H

#include "DeviceIndexTableModel.h"
#include "base/DeviceIndexEnabled.h"

class DeviceEnableTableModel : public DeviceIndexTableModel
{
    Q_OBJECT
public:
    enum Columns {
        COL_ENABLE,
        COL_DEVICE_ID,
        COL_DEVICE_SERIAL,
        COL_TOTAL
    };

    explicit DeviceEnableTableModel(QObject *parent = nullptr)
        : DeviceIndexTableModel(parent) {}

    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    int columnCount(const QModelIndex &parent = QModelIndex()) const override;

    Qt::ItemFlags flags(const QModelIndex & index) const override;
    QVariant data(const QModelIndex &index, int role) const override;
    QVariant headerData(int section, Qt::Orientation orientation,
                                int role = Qt::DisplayRole) const override;
    bool setData(const QModelIndex& index, const QVariant& value, int role) override;

    DeviceIndex deviceIndexOf(int row) const override;
    int rowOf(const DeviceIndex &deviceIndex) const override;

    QMap<int, bool> defaultColumnState() const override;

signals:
    void deviceListChanged(const DeviceIndexEnabledMap &new_devices);

public slots:
    void setDeviceList(const DeviceIndexEnabledMap &map);

protected:
    DeviceIndexEnabledMap devices;
};

#endif // DEVICEENABLETABLEMODEL_H
