/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "FileDialog.h"
#include "base/Globals.h"

QString FileDialog::getOpenFileName(QWidget *parent,
                                    const QString &caption,
                                    const QString &dir,
                                    const QString &filter,
                                    QString *selectedFilter)
{
    QFileDialog::Options options = QFileDialog::ReadOnly;
    if (Globals::instance().getUseQtDialogs())
        options |= QFileDialog::DontUseNativeDialog;
    return QFileDialog::getOpenFileName(parent, caption, dir, filter, selectedFilter, options);
}

QString FileDialog::getSaveFileName(QWidget *parent,
                                    const QString &caption,
                                    const QString &dir,
                                    const QString &filter,
                                    QString *selectedFilter)
{
    auto options = Globals::instance().getUseQtDialogs() ? QFileDialog::DontUseNativeDialog
                                                         : QFileDialog::Options();
    return QFileDialog::getSaveFileName(parent, caption, dir, filter, selectedFilter, options);
}

QString FileDialog::getExistingDirectory(QWidget *parent,
                                         const QString &caption,
                                         const QString &dir)
{
    QFileDialog::Options options = QFileDialog::ShowDirsOnly;
    if (Globals::instance().getUseQtDialogs())
        options |= QFileDialog::DontUseNativeDialog;
    return QFileDialog::getExistingDirectory(parent, caption, dir, options);
}
