#include "ClockDividerControlModule.h"

#include "regio/regop.h"
#include "sdb/AfiSdbId.h"
#include "base/DeviceIndex.h"

namespace {
enum {
    REG_CTRL                = 0x02,
    REG_STATUS              = 0x03,
};

union status_reg_t {
    uint16_t all;
    struct {
        unsigned adcPhaseOk: 1;
        unsigned unused_14: 14;
        unsigned tmValid: 1;
    } bit;
};

const quint16 REG_CONTROL_BIT_RESET          = 0x0001;
} // namespace

QList<AFI_SDB_ID> ClockDividerControlModule::getSupportedSdbIds()
{
    return {AFI_SDB_Clock_Divider_Contr};
}

std::optional<ClockDividerStatus> ClockDividerControlModule::readStatus()
{
    status_reg_t statusReg;
    RegOpVector r;
    r.RegRead16(REG_STATUS, &statusReg.all);

    if (!regOpExecRebased(r))
        return {};

    ClockDividerStatus st;
    st.adcPhaseOk = statusReg.bit.adcPhaseOk;
    st.tmValid = statusReg.bit.tmValid;
    return st;
}

bool ClockDividerControlModule::setResetEn(bool en)
{
    RegOpVector r;
    r.RegWrite16(REG_CTRL, en ? REG_CONTROL_BIT_RESET : 0);
    return regOpExecRebased(r);
}
