/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "DeviceModuleMapper.h"
#include "mregdev/qtmregdevice.h"
#include "sdb/SdbTypes.h"

void DeviceModuleMapper::initModules()
{
    for (auto it = modules.begin(); it != modules.end(); ++it) {
        auto p = *it;
        p->init();
    }
}

void DeviceModuleMapper::deinitModules()
{
    for (auto it = modules.begin(); it != modules.end(); ++it) {
        auto p = *it;
        p->deinit();
    }
    modules.clear();
}

QList<SdbDevice> DeviceModuleMapper::getModuleList(const QList<AFI_SDB_ID> &search) const
{
    return qtMregDevice->getSdb().filterDevices(search);
}

QList<SdbModuleIndex> DeviceModuleMapper::getModuleIndexes() const
{
    QList<SdbModuleIndex> list;
    for (const auto &i : modules) {
        SdbModuleIndex index;
        index.id = i->getSdbId();
        index.baseAddr = i->getBaseAddr();
        list.push_back(index);
    }
    return list;
}

QSharedPointer<AbstractDeviceModule> DeviceModuleMapper::findModule(const SdbModuleIndex index)
{
    for (const auto &i : qAsConst(modules)) {
        bool addr_match = (index.baseAddr == 0) || (index.baseAddr == i->getBaseAddr());
        bool id_match = (index.id == 0) || (index.id == i->getSdbId());
        if (addr_match && id_match)
            return i;
    }
    return {};
}

QSharedPointer<const AbstractDeviceModule> DeviceModuleMapper::findModule(const SdbModuleIndex index) const
{
    for (const auto &i : qAsConst(modules)) {
        bool addr_match = (index.baseAddr == 0) || (index.baseAddr == i->getBaseAddr());
        bool id_match = (index.id == 0) || (index.id == i->getSdbId());
        if (addr_match && id_match)
            return i;
    }
    return {};
}

QList<QSharedPointer<AbstractDeviceModule>> DeviceModuleMapper::findModules(AFI_SDB_ID id)
{
    QList<QSharedPointer<AbstractDeviceModule>> result;
    for (auto &i : modules) {
        if (id == i->getSdbId())
            result.push_back(i);
    }
    return result;
}

QList<QSharedPointer<const AbstractDeviceModule>> DeviceModuleMapper::findModules(AFI_SDB_ID id) const
{
    QList<QSharedPointer<const AbstractDeviceModule>> result;
    for (const auto &i : qAsConst(modules)) {
        if (id == i->getSdbId())
            result.push_back(i);
    }
    return result;
}
