/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "FlashProgStatus.h"

QString FlashProgStatus::toString() const
{
    QString str;
    if (!chip_valid)
        return {};
    if (taskName.isNull()) {
        str += QString("%1Mb").arg(chip_capacity_Mib);
        if(addr4bMode)
            str += " (4B mode)";
    } else {
        if (taskRunning)
            str = QString("%1 %2%").arg(taskName).arg((int) (100.0 * progress));
        else if (taskAborted)
            str = QString("%1 error at %2%").arg(taskName).arg((int) (100.0 * progress));
        else
            str = QString("%1 Ok").arg(taskName);
    }
    return str;
}

Severity FlashProgStatus::status() const
{
    if (!chip_valid)
        return Severity::Error;
    if (taskAborted)
        return Severity::Critical;
    if (taskRunning)
        return Severity::Warning;
    return Severity::Normal;
}
