#include "MscCoreConfigConverter.h"

#include "base/DeviceIndexConverter.h"
#include "base/ConfigConverterUtil.h"
#include "device-modules/MscHist/MscHistConfigConverter.h"
#include "device-modules/MscInput/MscInputConfigConverter.h"
#include "device-modules/MscStream/MscStreamConfigConverter.h"

#include "MscCoreConfig.h"

namespace {
constexpr const char *k_mstream = "mstream";
constexpr const char *k_hist = "hist";
constexpr const char *k_input = "input";
constexpr const char *k_chThreshold = "chThreshold";
constexpr const char *k_chPolarity = "chPolarity";
} // namespace

template<>
bool AbstractConfigConverter<MscCoreConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<MscCoreConfig>::toJsonObject(const MscCoreConfig &in)
{
    QVariantMap m;
    m[k_mstream] = QVariant::fromValue(in.mstream);
    m[k_hist] = QVariant::fromValue(in.hist);
    m[k_input] = ConfigConverterUtil::toJsonValue(in.input);
    m[k_chThreshold] = ConfigConverterUtil::toJsonValue<int, int>(in.chThreshold);
    m[k_chPolarity] = ConfigConverterUtil::toJsonValue<int, bool>(in.chPolarity);
    return fromVariantMap(m);
}

template<>
MscCoreConfig AbstractConfigConverter<MscCoreConfig>::fromJsonObject(const QJsonObject &m)
{
    MscCoreConfig out;
    update_value(m, k_mstream, &out.mstream);
    update_value(m, k_hist, &out.hist);
    if(m.contains(k_input))
        out.input = ConfigConverterUtil::toMap<int, MscInputChConfig>(m[k_input]);
    if(m.contains(k_chThreshold))
        out.chThreshold = ConfigConverterUtil::toMap<int, int>(m[k_chThreshold]);
    if(m.contains(k_chPolarity))
        out.chPolarity = ConfigConverterUtil::toMap<int, bool>(m[k_chPolarity]);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<MscCoreConfig>::toJsonValue(const MscCoreConfig &in)
{
    return toJsonObject(in);
}

template<>
MscCoreConfig AbstractConfigConverter<MscCoreConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<MscCoreConfig>::registerSubTypes()
{
    AbstractConfigConverter<DeviceIndex>::registerTypes();
    AbstractConfigConverter<MscStreamConfig>::registerTypes();
    AbstractConfigConverter<MscHistConfig>::registerTypes();
    AbstractConfigConverter<MscInputChConfig>::registerTypes();
}
