/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef NETWORKPORTMODULEREGISTERS_H
#define NETWORKPORTMODULEREGISTERS_H

enum NetworkPortModuleRegisters {
    // PHY
    REG_ETH_PORT_PHY_BASE              = 0x000,
    REG32_ETH_PORT_PHY_MAGIC           = 0x000,
//    REG32_ETH_PORT_PHY_CONTROL         = 0x002, // unused
    REG32_ETH_PORT_PHY_STATUS          = 0x004,
    // PCS/PMA
    REG_ETH_PORT_PCSPMA_BASE           = 0x040,
    REG32_ETH_PORT_PCSPMA_MAGIC        = 0x000,
//    REG32_ETH_PORT_PCSPMA_CONTROL      = 0x002, // unused
    REG32_ETH_PORT_PCSPMA_STATUS       = 0x004,
    REG32_ETH_PORT_PCSPMA_COUNTER_0    = 0x010,
    REG32_ETH_PORT_PCSPMA_TIMER_0      = 0x020,
    // MAC
    REG_ETH_PORT_MAC_BASE            = 0x080,
    REG32_ETH_PORT_MAC_MAGIC         = 0x000,
//    REG32_ETH_PORT_MAC_CONTROL       = 0x002, // unused
    REG32_ETH_PORT_MAC_STATUS        = 0x004,
    REG32_ETH_PORT_MAC_MTU           = 0x006,
    REG32_ETH_PORT_MAC_COUNTER_0     = 0x020,
    // HWIP 100-17F
    REG_ETH_PORT_HWIP_BASE             = 0x100,
    REG32_ETH_PORT_HWIP_MAGIC          = 0x000,
//    REG32_ETH_PORT_HWIP_CONTROL        = 0x002, // unused
    REG32_ETH_PORT_HWIP_STATUS         = 0x004,
    REG32_ETH_PORT_HWIP_HOST_IP        = 0x006,
    REG64_ETH_PORT_HWIP_HOST_MAC       = 0x008,
    REG32_ETH_PORT_HWIP_REGIO_ERROR_ADDR = 0x00C,
    REG32_ETH_PORT_HWIP_REGIO_ERROR_CODE = 0x00E,
    REG32_ETH_PORT_HWIP_MLINK_CONFIG   = 0x010,
    REG32_ETH_PORT_HWIP_MASTER_IP      = 0x012,
    REG_ETH_PORT_HWIP_MASTER_PORT      = 0x014,
    //REG32_ETH_PORT_HWIP_MSTREAM_CONFIG = 0x018,
    REG32_ETH_PORT_HWIP_MSTREAM_IP     = 0x01A,
    REG_ETH_PORT_HWIP_MSTREAM_PORT     = 0x01C,
    REG32_ETH_PORT_HWIP_COUNTER_0      = 0x040,
    REG32_ETH_PORT_HWIP_TIMER_0        = 0x050,
    // MSTREAM 180-1FF
    REG_ETH_PORT_MSTREAM_BASE             = 0x180,
    REG32_ETH_PORT_MSTREAM_MAGIC          = 0x000,
//    REG32_ETH_PORT_MSTREAM_CONTROL        = 0x002, // unused
    REG32_ETH_PORT_MSTREAM_STATUS         = 0x004,
    REG_ETH_PORT_MSTREAM_MTU_MIN          = 0x006,
    REG_ETH_PORT_MSTREAM_MTU_MAX          = 0x007,
    REG_ETH_PORT_MSTREAM_MTU              = 0x008,
    REG_ETH_PORT_MSTREAM_MAX_PACKET_SIZE  = 0x009,
    REG32_ETH_PORT_MSTREAM_DEFRAG_TIMEOUT = 0x00A, // RW
    REG32_ETH_PORT_MSTREAM_RETX_TIME      = 0x010,
    REG32_ETH_PORT_MSTREAM_RETX_CONFIG    = 0x012,
    REG32_ETH_PORT_MSTREAM_RETX_EST_RTT   = 0x014,
    REG32_ETH_PORT_MSTREAM_CLK_KHZ        = 0x01E,
    REG32_ETH_PORT_MSTREAM_COUNTER_0      = 0x020,
    REG32_ETH_PORT_MSTREAM_TIMER_0        = 0x040,
    // DESCR
    REG_ETH_PORT_DESCR_BASE             = 0x000,
    REG32_ETH_PORT_DESCR_MAGIC          = 0x000,
//    REG32_ETH_PORT_DESCR_CONTROL        = 0x002, // unused
//    REG32_ETH_PORT_DESCR_STATUS         = 0x004, // unused
    REG_ETH_PORT_DESCR_CAPABILITY       = 0x006,
    REG_ETH_PORT_DESCR_PORT_ID          = 0x007,
    REG_ETH_PORT_DESCR_PHY_REG_BASE     = 0x008,
    REG_ETH_PORT_DESCR_PCS_PMA_REG_BASE = 0x009,
    REG_ETH_PORT_DESCR_MAC_REG_BASE     = 0x00A,
    REG_ETH_PORT_DESCR_HWIP_REG_BASE    = 0x00B,
    REG_ETH_PORT_DESCR_MSTREAM_REG_BASE = 0x00C,
};

#endif // NETWORKPORTMODULEREGISTERS_H
