#include "SpillConfigConverter.h"

#include "base/ConfigConverterUtil.h"

#include "SpillConfig.h"

namespace {
constexpr const char *k_durPhase1 = "durPhase1";
constexpr const char *k_durPhase2 = "durPhase2";
constexpr const char *k_durPhase3 = "durPhase3";
constexpr const char *k_durPhase4 = "durPhase4";
constexpr const char *k_durPhase5 = "durPhase5";

constexpr const char *k_source = "source";

constexpr const char *k_softSpill = "softSpill";
constexpr const char *k_timerPeriod = "timerPeriod";
}

template<>
bool AbstractConfigConverter<SpillConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<SpillConfig>::toJsonObject(const SpillConfig &in)
{
    QVariantMap m;

    m[k_durPhase1] = QVariant::fromValue(in.durPhase1);
    m[k_durPhase2] = QVariant::fromValue(in.durPhase2);
    m[k_durPhase3] = QVariant::fromValue(in.durPhase3);
    m[k_durPhase4] = QVariant::fromValue(in.durPhase4);
    m[k_durPhase5] = QVariant::fromValue(in.durPhase5);

    m[k_source] = QVariant::fromValue(static_cast<int>(in.source));

    m[k_softSpill] = QVariant::fromValue(in.softSpill);
    m[k_timerPeriod] = QVariant::fromValue(in.timerPeriod);

    return fromVariantMap(m);
}

template<>
SpillConfig AbstractConfigConverter<SpillConfig>::fromJsonObject(const QJsonObject &in)
{
    SpillConfig out;

    update_value(in, k_durPhase1, &out.durPhase1);
    update_value(in, k_durPhase2, &out.durPhase2);
    update_value(in, k_durPhase3, &out.durPhase3);
    update_value(in, k_durPhase4, &out.durPhase4);
    update_value(in, k_durPhase5, &out.durPhase5);

    if(in.contains(k_source)) {
        int source;
        update_value(in, k_source, &source);
        out.source = static_cast<SpillConfig::spill_sel_t>(source);
    }

    update_value(in, k_softSpill, &out.softSpill);
    update_value(in, k_timerPeriod, &out.timerPeriod);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<SpillConfig>::toJsonValue(const SpillConfig &in)
{
    return toJsonObject(in);
}

template<>
SpillConfig AbstractConfigConverter<SpillConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<SpillConfig>::registerSubTypes()
{
}
