/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef CLOCKCONTROLDIALOG_H
#define CLOCKCONTROLDIALOG_H

#include <QDialog>
#include <optional>

#include "device-modules/ClockControl/ClockControlConfig.h"
#include "device-modules/ClockControl/ClockStatus.h"

struct ClockBuildParams;
class ClockControlWidget;
class DeviceIndex;
class DeviceIndexEnabledMap;

class ClockControlDialog : public QDialog
{
    Q_OBJECT
public:
    explicit ClockControlDialog(QWidget *parent = nullptr);

signals: void clockControlConfigChanged(const ClockControlConfig &config) const;
    void setClockControlBp(const DeviceIndex &index, const ClockBuildParams &bp);
    void setDeviceList(const DeviceIndexEnabledMap &map);
    void setClockControlStatus(const DeviceIndex &index, const std::optional<ClockStatus> &st);
    void setClockControlConfig(const ClockControlConfig &config);

private slots:
    void restoreDefaults();

private:
    ClockControlWidget * const widget;
};

#endif // CLOCKCONTROLDIALOG_H
