#ifndef MODULARDEVICESTATUSMODEL_H
#define MODULARDEVICESTATUSMODEL_H

#include "mvc/DeviceIndexTableModel.h"

#include "../WrStatus/WrStatusTableHelper.h"

struct DeviceIndexEnabledMap;

class DeviceIndex;
class ModularDeviceStatus;
class ModularDeviceStatusModelPrivate;
class DeviceStatusBaseModelPrivateEntry;

class ModularDeviceStatusModel : public DeviceIndexTableModel
{
    Q_OBJECT
public:
    enum Columns {COL_DSB_INDEX, COL_DSB_SERIAL,
                   COL_DSB_FW, COL_DSB_BMC,
                   COL_DSB_TEMP,
                   COL_DSB_FLASH,
                   COL_DSB_RUNLOGIC,
                   COL_CLOCK_STATUS,
                   COL_DES_STATUS,
                   COL_TRIG_ACCEPT, COL_TRIG_BLOCK, COL_TRIG_DISCARD, COL_TRIG_VETO,
                   COL_TRIG_ACCEPT_D, COL_TRIG_BLOCK_D, COL_TRIG_DISCARD_D, COL_TRIG_VETO_D,
                   COL_TRIG_QUEUE,
                   COL_DEV_BUSY_BITS,
                   COL_DEV_BUSY_TIME,
                   COL_EV_NUM, COL_EV_TS,
                   // HptdcStatus
                   COL_HPTDC_STATUS,
                   COL_WAVEREC,
                   // WrStatus
                   COL_DSB_WR_START,
                   COL_DSB_WR_END = COL_DSB_WR_START + WrStatusTableHelper::COL_TOTAL - 1,

                   // COL_DSB_BUSY,COL_DSB_IDLE,COL_DSB_OCCUPANCY,
                   COL_DSB_TOTAL
    };

    ModularDeviceStatusModel(QObject *parent = nullptr);
    ~ModularDeviceStatusModel() override;

    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    virtual int columnCount(const QModelIndex &parent = QModelIndex()) const override;

    Qt::ItemFlags flags(const QModelIndex & index) const override;
    virtual QVariant data(const QModelIndex &index, int role) const override;
    virtual QVariant headerData(int section, Qt::Orientation orientation,
                                int role = Qt::DisplayRole) const override;
//    bool setData(const QModelIndex& index, const QVariant& value, int role) override;
    DeviceIndex deviceIndexOf(int row) const override;
    int rowOf(const DeviceIndex &sourceIndex) const override;

    QMap<int, bool> defaultColumnState() const override;

public slots:
    void setStatus(const ModularDeviceStatus &status);
    void setMultiStatus(const QMap<DeviceIndex, ModularDeviceStatus> &statusMap);

signals:
public slots:
    void setDeviceList(const DeviceIndexEnabledMap &map);

private:
    void invalidateRow(const DeviceIndex &index);
    QVariant data_BackgroundRole(const DeviceStatusBaseModelPrivateEntry &index, int column) const;
    QVariant data_DisplayRole(const DeviceStatusBaseModelPrivateEntry &device, int column) const;
    QVariant data_UserRole(const DeviceIndex &index, int column) const;
    QVariant data_ToolTipRole(const DeviceStatusBaseModelPrivateEntry &device, int column) const;

    Q_DECLARE_PRIVATE(ModularDeviceStatusModel)
    ModularDeviceStatusModelPrivate * const d_ptr;
};

#endif // MODULARDEVICESTATUSMODEL_H
