#ifndef FLASHDEVTYPES_H
#define FLASHDEVTYPES_H

#include <cstdint>

constexpr const uint32_t REG_CTRL = 0x2;
constexpr const uint32_t REG_CMD = 0x5;
constexpr const uint32_t REG_READBACK = 0x6;
constexpr const uint32_t REG32_DATA_ADDR = 0x8;
constexpr const uint32_t REG_STAT_1 = 0xA;
constexpr const uint32_t REG_STAT_2 = 0xB;
constexpr const uint32_t REG_FLASH_ADDR = 0xfe;
constexpr const uint32_t REG_DATA = 0x80;

enum FlashCommand : uint16_t {
    CMD_WRITE = 0x02,
    CMD_READ = 0x03,
    CMD_SE = 0xd8, // Sector erase
    CMD_BE = 0xc7, // Bulk erase
    CMD_WRITE_EN = 0x06,
    CMD_WRVECR = 0x61,
    CMD_RDSR = 0x05,
    CMD_RDID = 0x9f, // JEDEC ID
    CMD_ADDR4B_EN = 0xb7,
    CMD_ADDR4B_DIS = 0xe9,
};

enum : uint16_t {
    REG_CTRL_BIT_ADDR4B_EN   = 1 << 15,
};

constexpr const int JEDEC_ID_OK = 0x20;

constexpr const int SECTOR_ERASE_TIMEOUT = 1300;
constexpr const int BULK_ERASE_TIMEOUT = 200000;
constexpr const int WRITE_TIMEOUT_MS = 100;
constexpr const int READ_TIMEOUT_MS = 100;

#endif // FLASHDEVTYPES_H
