/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef FPGABITFILE_H
#define FPGABITFILE_H

#include <QByteArray>
#include <QDateTime>
#include <QMap>

struct FpgaBitFile
{
    static constexpr const uint32_t BITSTREAM_SYNC = 0x665599AA;

    FpgaBitFile() = default;
    FpgaBitFile(const FpgaBitFile &) = default;
    FpgaBitFile &operator=(const FpgaBitFile &) = default;

    static int findBitstreamSync(const QByteArray &data);
    static void dumpBitstream(const QByteArray &bitstream);

    bool readFile(const QString fileName);
    QMap<QString, QString> getMeta() const { return meta; }
    QString getProject() const;
    QString getPart() const;
    QDateTime getDateTime() const;
    QByteArray getBitstream() const { return bitstream; }

private:
    bool parseBitFile(const QByteArray &bitfileData);

    QMap<QString, QString> meta;
    QByteArray bitstream;
    int syncPos = 0;
};

#endif // FPGABITFILE_H
