#ifndef BASEAPPCORE_H
#define BASEAPPCORE_H

#include <QObject>

#include "base/RcRunId.h"
#include "fsm/FsmState.h"
#include "pnp-server/PNPMessage.h"
#include "remote-control-server/RcProgramState.h"

class BaseFsm;
class ConfigIndexName;
class ProgramDescription;
class RemoteControlServer;
class RemoteControlServerStatus;
struct RunDocument;

class BaseAppCore : public QObject
{
    Q_OBJECT
public:
    explicit BaseAppCore(QObject *parent = nullptr);
    virtual ~BaseAppCore() override;

signals:
    void closeProgram(const ProgramDescription &) const;
    void updateProgramDescription(const ProgramDescription &) const;
    void rcProgramStateChanged(const RcProgramState &) const;
    void closeAll() const;
    void runStateChanged(bool run) const;
    void switch_to_config(const ConfigIndexName &conf) const; // unused just to subscribe
    void sendRunError(const RcRunId &runId, QString text = {}) const;
    void sendRunWarning(const RcRunId &runId, QString text) const;
    void allowRemoteControl(bool) const;
    void sendRunDocumentToRc(const RunDocument &rd) const;

public slots:
    virtual void setRunState(bool) {}
    virtual void configSelectorChanged(const ConfigIndexName &conf);
    virtual void onRunError(const RcRunId &runId, QString text);
    virtual void onRunWarning(const RcRunId &runId, QString text);
    virtual void remoteStatusUpdated(const RemoteControlServerStatus &status);
    virtual void onFsmStateChange(const FsmState &state, const FsmState &oldState);

protected:
    virtual bool isRun() const = 0;
    virtual const BaseFsm *getFsm() const = 0;

public: //protected:
    RemoteControlServer * const remoteControlServer;
    bool underRemoteCtrl = false;
    ProgramIndex programIndex;
    ProgramDescription progDescr;
    RcRunId rcRunId;
    RcProgramState rcProgramState;
};

#endif // BASEAPPCORE_H
