/*
**    Copyright 2021 Ilia Slepnev, Ivan Filippov
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef BASECONFIGDISPATCHER_H
#define BASECONFIGDISPATCHER_H

#include "base/AbstractDispatcher.h"
#include "base/ConfigKey.h"

struct Ad5622Config;
class AdcSerDesDelayMap;
class BaseConfigDispatcherPrivate;
class BaseDeviceAppConfig;
class ClientIndex;
class ConfigIndexName;
class DeviceIndex;
class DeviceIndexEnabledMap;
struct DlnpCalibrationTriggerConfig;
class FirFilterConfig;
class ModularDeviceConfig;
class ModularMregDevice;
struct AdcLemoMuxCtrlConfig;
struct ClockControlConfig;
struct CrosspointSwitchConfig;
class NetworkPortConfig;
struct SparseParams;
class RunLogicConfig;
class TrigCtrlStatConfig;
class TtlIoConfig;
class WaveBlcConfig;
class WaveRecConfig;
struct ZmqConfig;

class BaseConfigDispatcher : public AbstractDispatcher
{
    Q_OBJECT
public:
    explicit BaseConfigDispatcher(QObject *parent = nullptr);
    virtual ~BaseConfigDispatcher() override;

    virtual void sendCache();

    template<class T>
    void subscribeConfig(T *obj) {
        connect(obj, &T::configChanged,
                this, &BaseConfigDispatcher::setConfig);
        connect(this, &BaseConfigDispatcher::configChanged,
                obj, &T::setConfig);
        logConnection(obj, "AppConfig");
        emit configChanged(config_key, getAppConfigCache()); // FIXME: is it necessary?
    }

    void forwardDeviceConfig(const ModularMregDevice *obj);

    template<class T>
    void subscribeCS(T *obj) {
            connect(this, &BaseConfigDispatcher::switch_to_config,
                    obj, &T::configSelectorChanged,
                    Qt::UniqueConnection);

            connect(obj, &T::switch_to_config,
                    this, &BaseConfigDispatcher::configSelectorChanged,
                    Qt::UniqueConnection);
            logConnection(obj, "ConfigIndexName");
    }

    template<class T>
    void subscribeClockControlConfig(T *obj) {
        connect(this, &BaseConfigDispatcher::clockControlConfigChanged,
                obj, &T::setClockControlConfig,
                Qt::UniqueConnection);

        connect(obj, &T::clockControlConfigChanged,
                this, &BaseConfigDispatcher::setClockControlConfig,
                Qt::UniqueConnection);
        logConnection(obj, "ClockControlConfig");
    }

    template<class T>
    void subscribeCrosspointSwitchConfig(T *obj) {
        connect(this, &BaseConfigDispatcher::crosspointSwitchConfigChanged,
                obj, &T::setCrosspointSwitchConfig,
                Qt::UniqueConnection);

        connect(obj, &T::crosspointSwitchConfigChanged,
                this, &BaseConfigDispatcher::setCrosspointSwitchConfig,
                Qt::UniqueConnection);
        logConnection(obj, "CrosspointSwitchConfig");
    }

    template<class T>
    void subscribePreamplifierConfig(T *obj) {
        connect(this, &BaseConfigDispatcher::preamplifierConfigChanged,
                obj, &T::setPreamplifierConfig,
                Qt::UniqueConnection);

        connect(obj, &T::preamplifierConfigChanged,
                this, &BaseConfigDispatcher::setPreamplifierConfig,
                Qt::UniqueConnection);
        logConnection(obj, "PreamplifierConfig");
    }

    template<class T>
    void subscribeDlnpCalibrationTriggerConfig(T *obj) {
        connect(this, &BaseConfigDispatcher::dlnpCalibrationTriggerConfigChanged,
                obj, &T::setDlnpCalibrationTriggerConfig,
                Qt::UniqueConnection);

        connect(obj, &T::dlnpCalibrationTriggerConfigChanged,
                this, &BaseConfigDispatcher::setDlnpCalibrationTriggerConfig,
                Qt::UniqueConnection);
        logConnection(obj, "DlnpCalibrationTriggerConfig");
    }

    template<class T>
    void subscribeFirFilterConfig(T *obj) {
        connect(this, &BaseConfigDispatcher::firFilterConfigChanged,
                obj, &T::setFirFilterConfig,
                Qt::UniqueConnection);

        connect(obj, &T::firFilterConfigChanged,
                this, &BaseConfigDispatcher::setFirFilterConfig,
                Qt::UniqueConnection);
        logConnection(obj, "FirFilterConfig");
    }

    template<class T>
    void subscribeAdcLemoMuxConfig(T *obj) {
        connect(this, &BaseConfigDispatcher::adcLemoMuxConfigChanged,
                obj, &T::setAdcLemoMuxConfig,
                Qt::UniqueConnection);

        connect(obj, &T::adcLemoMuxConfigChanged,
                this, &BaseConfigDispatcher::setAdcLemoMuxConfig,
                Qt::UniqueConnection);
        logConnection(obj, "AdcLemoMuxConfig");
    }

    template<class T>
    void subscribeNetworkPortConfig(T *obj) {
        connect(this, &BaseConfigDispatcher::networkPortConfigChanged,
                obj, &T::setNetworkPortConfig,
                Qt::UniqueConnection);

        connect(obj, &T::networkPortConfigChanged,
                this, &BaseConfigDispatcher::setNetworkPortConfig,
                Qt::UniqueConnection);
        logConnection(obj, "NetworkPortConfig");
    }

    template<class T>
    void subscribeSerDesDelay(T *obj) {
        connect(obj, &T::devSerDesDelayChanged,
                this, &BaseConfigDispatcher::setDevSerDesDelay,
                Qt::UniqueConnection);
        connect(this, &BaseConfigDispatcher::devSerDesDelayChanged,
                obj, &T::setDevSerDesDelay,
                Qt::UniqueConnection);
        logConnection(obj, "AdcSerDesDelayMap");
    }

    template<class T>
    void subscribeRunLogicConfig(T *obj) {
        connect(this, &BaseConfigDispatcher::runLogicConfigChanged,
                obj, &T::setRunLogicConfig,
                Qt::UniqueConnection);

        connect(obj, &T::runLogicConfigChanged,
                this, &BaseConfigDispatcher::setRunLogicConfig,
                Qt::UniqueConnection);
        logConnection(obj, "RunLogicConfig");
    }

    template<class T>
    void subscribeSparseParams(T *obj) {
        connect(this, &BaseConfigDispatcher::sparseParamsChanged,
                obj, &T::setSparseParams,
                Qt::UniqueConnection);

        connect(obj, &T::sparseParamsChanged,
                this, &BaseConfigDispatcher::setSparseParams,
                Qt::UniqueConnection);
        logConnection(obj, "SparseParams");
    }

    template<class T>
    void subscribeTtlCfg(T *obj) {
        connect(this, &BaseConfigDispatcher::ttlCfgChanged,
                obj, &T::setTtlCfg,
                Qt::UniqueConnection);

        connect(obj, &T::ttlCfgChanged,
                this, &BaseConfigDispatcher::setTtlCfg,
                Qt::UniqueConnection);
        logConnection(obj, "TtlCfg");
    }

    template<class T>
    void subscribeTrigCtrlStatConfig(T *obj) {
        connect(this, &BaseConfigDispatcher::trigCtrlStatConfigChanged,
                obj, &T::setTrigCtrlStatConfig,
                Qt::UniqueConnection);

        connect(obj, &T::trigCtrlStatConfigChanged,
                this, &BaseConfigDispatcher::setTrigCtrlStatConfig,
                Qt::UniqueConnection);
        logConnection(obj, "TrigCtrlStatConfig");
    }

    template<class T>
    void subscribeWaveBlcConfig(T *obj) {
        connect(this, &BaseConfigDispatcher::waveBlcConfigChanged,
                obj, &T::setWaveBlcConfig,
                Qt::UniqueConnection);

        connect(obj, &T::waveBlcConfigChanged,
                this, &BaseConfigDispatcher::setWaveBlcConfig,
                Qt::UniqueConnection);
        logConnection(obj, "WaveBlcConfig");
    }

    template<class T>
    void subscribeWaveRecConfig(T *obj) {
        connect(this, &BaseConfigDispatcher::waveRecConfigChanged,
                obj, &T::setWaveRecConfig,
                Qt::UniqueConnection);

        connect(obj, &T::waveRecConfigChanged,
                this, &BaseConfigDispatcher::setWaveRecConfig,
                Qt::UniqueConnection);
        logConnection(obj, "WaveRecConfig");
    }

    template<class T>
    void subscribeAllowRemoteControl(T *obj) {
        connect(obj, &T::allowRemoteControlChanged,
                this, &BaseConfigDispatcher::setAllowRemoteControl,
                Qt::UniqueConnection);
        connect(this, &BaseConfigDispatcher::allowRemoteControlChanged,
                obj, &T::setAllowRemoteControl,
                Qt::UniqueConnection);
        logConnection(obj, "AllowRemoteControl");
    }

    template<class T>
    void subscribeZmqConfig(T *obj) {
        connect(obj, &T::zmqConfigChanged,
                this, &BaseConfigDispatcher::setZmqConfig,
                Qt::UniqueConnection);
        connect(this, &BaseConfigDispatcher::zmqConfigChanged,
                obj, &T::setZmqConfig,
                Qt::UniqueConnection);
        logConnection(obj, "ZmqConfig");
    }

public:
    template<class T>
    void subscribeErrOnTrigOnXOff(T *obj) {
        connect(obj, &T::errOnTrigOnXOffChanged,
                this, &BaseConfigDispatcher::setErrOnTrigOnXOff,
                Qt::UniqueConnection);
        connect(this, &BaseConfigDispatcher::errOnTrigOnXOffChanged,
                obj, &T::setErrOnTrigOnXOff,
                Qt::UniqueConnection);
        logConnection(obj, "ErrOnTrigOnXOff");
    }
signals:    void errOnTrigOnXOffChanged(bool v) const;
public slots: void setErrOnTrigOnXOff(bool isErr);

public:
    template<class T>
    void subscribeErrOnSerDesUnlock(T *obj) {
        connect(obj, &T::errOnSerDesUnlockChanged,
                this, &BaseConfigDispatcher::setErrOnSerDesUnlock,
                Qt::UniqueConnection);
        connect(this, &BaseConfigDispatcher::errOnSerDesUnlockChanged,
                obj, &T::setErrOnSerDesUnlock,
                Qt::UniqueConnection);
        logConnection(obj, "ErrOnTrigOnXOff");
    }
signals:    void errOnSerDesUnlockChanged(bool v) const;
public slots: void setErrOnSerDesUnlock(bool isErr);

signals:
    void configKeyChanged(const ConfigKey &key) const;
    void clientIndexChanged(const ClientIndex &key) const;
    void switch_to_config(const ConfigIndexName &conf) const;
    void deviceListChanged(const DeviceIndexEnabledMap &) const;
    void dlnpCalibrationTriggerConfigChanged(const DlnpCalibrationTriggerConfig &) const;
    void firFilterConfigChanged(const FirFilterConfig &) const;
    void adcLemoMuxConfigChanged(const AdcLemoMuxCtrlConfig &) const;
    void clockControlConfigChanged(const ClockControlConfig &config) const;
    void crosspointSwitchConfigChanged(const CrosspointSwitchConfig &) const;
    void networkPortConfigChanged(const NetworkPortConfig &) const;
    void preamplifierConfigChanged(const Ad5622Config &) const;
    void runLogicConfigChanged(const RunLogicConfig &) const;
    void sparseParamsChanged(const SparseParams &) const;
    void trigCtrlStatConfigChanged(const TrigCtrlStatConfig &) const;
    void ttlCfgChanged(const TtlIoConfig &) const;
    void waveBlcConfigChanged(const WaveBlcConfig &) const;
    void waveRecConfigChanged(const WaveRecConfig &) const;
    void devSerDesDelayChanged(const DeviceIndex &index, const AdcSerDesDelayMap &delay) const;
    void allowRemoteControlChanged(bool allow) const;
    void deviceConfigChanged(const ConfigKey &key, const ModularDeviceConfig &) const;
    // void configChanged(const BaseDeviceAppConfig &) const;
    void configChanged(const ConfigKey &key, const BaseDeviceAppConfig &) const;
    void zmqConfigChanged(const ZmqConfig &zmqCfg) const;

public slots:
    void configSelectorChanged(const ConfigIndexName &cin);
    virtual void setDeviceList(const DeviceIndexEnabledMap &map);
    virtual void setDlnpCalibrationTriggerConfig(const DlnpCalibrationTriggerConfig &cfg);
    virtual void setFirFilterConfig(const FirFilterConfig &config);
    virtual void setAdcLemoMuxConfig(const AdcLemoMuxCtrlConfig &config);
    virtual void setClockControlConfig(const ClockControlConfig &config);
    virtual void setCrosspointSwitchConfig(const CrosspointSwitchConfig &config);
    virtual void setNetworkPortConfig(const NetworkPortConfig &config);
    virtual void setSparseParams(const SparseParams &config);
    virtual void setPreamplifierConfig(const Ad5622Config &config);
    virtual void setRunLogicConfig(const RunLogicConfig &);
    virtual void setTrigCtrlStatConfig(const TrigCtrlStatConfig &);
    virtual void setTtlCfg(const TtlIoConfig &);
    virtual void setWaveBlcConfig(const WaveBlcConfig &config);
    virtual void setWaveRecConfig(const WaveRecConfig &config);
    virtual void setDevSerDesDelay(const DeviceIndex &index, const AdcSerDesDelayMap &delay);
    virtual void setAllowRemoteControl(bool allow);
    virtual void setZmqConfig(const ZmqConfig &zmqCfg);
    virtual void setDeviceConfig(const DeviceIndex &index, const ModularDeviceConfig &config) {}
    virtual void setConfig(const BaseDeviceAppConfig &newConfig);

protected:
    void incrementConfigKey(bool sendFullCache = false);
    virtual void sendConfigKeyCache();
    virtual DeviceIndexEnabledMap getDeviceIndexEnabledMap() const = 0;
    virtual BaseDeviceAppConfig &getAppConfigCache();
    // virtual BaseDeviceAppConfig getAppConfigCache() const;
    void sendAdcSerDesCache();

    ConfigKey config_key;

private:
    Q_DECLARE_PRIVATE(BaseConfigDispatcher)
    BaseConfigDispatcherPrivate * const d_ptr;
};

#endif // BASECONFIGDISPATCHER_H
