//
//    Copyright 2014-2015 Ivan Filippov
//    Copyright 2014-2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef CLIENT_H
#define CLIENT_H

#include <QHostAddress>

#include "base/ClientIndex.h"

#include "RcClientData.h"
#include "RunStates.h"
#include "RcRunInfo.h"

#include "base/DnsLookup.h"
#include "config/RcClientProtoVersions.h"
#include "fsm/FsmState.h"
#include "remote-control-server/types.h"

class QTcpSocket;
class QTimer;
struct RunDocument;
struct RcProgramState;

Q_DECLARE_METATYPE(QHostAddress)


class Client : public QObject
{
    Q_OBJECT
public:
    Client(const ClientIndex &index);

    QString getIdentStr() const { return QString("Client '%1'")
                .arg(index.toString()); }
    QString getHostAddr() const { return hostAddr.toString(); }
    quint16 getHostPort() const { return hostPort; }

public slots:
    void sendRunState(FsmState state, const RcRunId &_runId, const QVariant &evNum);
    void startReadOut(const ClientIndex &i);
    void stopReadOut(const ClientIndex &i);
    void setEvNumCheck(const ClientIndex &i, bool evCheck);
    void deleteClient(const ClientIndex &i);
    void setAddress(const ClientIndex &i, const RcClientPeerInfo &p);
    void setEmptyAddress(const ClientIndex &i);
    void evNumRequest(int checkIndex);
    void sendRunDocumentToClients(const RunDocument &rd);
    void sendRcProgramState(const RcProgramState &allState) const;

private slots:
    void reconnectToServer();
    void readCommandResponce();
    void displayError(QAbstractSocket::SocketError socketError);
    void disconnected();
    void checkData();
    void reverseEntryUpdated(const DnsReverseEntry &ent);

signals:
    void statusUpdated(const ClientIndex &id, QString status) const;
    void fsmStateUpdated(const ClientIndex &id, FsmState state) const;
    void rcProgramStateUpdated(const ClientIndex &id, const RcProgramState &) const;
    void ready4Run(const ClientIndex &id, bool read4Run) const;
    void runError(const ClientIndex &id, const RcRunId &runId, QString errStr) const;
    void runWarning(const ClientIndex &id, const RcRunId &runId, QString errStr) const;
    void backPressure(const ClientIndex &id, const RcRunId &runId, bool backPressure) const;
    void vmeFlag(const ClientIndex &id, bool hasVme) const;
    void runStateUpdated(const ClientIndex &id, RUN_STATES state, const RcRunId &runId) const;
    void evNumResponse(int checkIndex, const ClientIndex &clientIndex, const EvNumSet &evNum) const;
    void config_transfer(const ClientIndex &index, const QJsonObject &json) const; // from client to top
    void protocolChanged(const ClientIndex &id, RC_CLIENT_PROTO_ID protoId) const;
    void statusChanged(const ClientIndex &id, const RcClientPeerInfo &) const;

private:
    void sendCommand(const QStringList &commandList);
    bool isConnectionAlive() const;
    bool processIncomingMsg();
    bool processIncomingMsgV4_5();
    bool command_line_parse(QByteArray data);
    void config_parse(QByteArray data);
    bool command_json_parse(const QByteArray &data);

    const ClientIndex index;
    bool enabled;
    bool performEvCheck = false;
    QTcpSocket * const tcpSocket;
    RcClientPeerInfo peer;
    QHostAddress hostAddr;
    quint16 hostPort;
    QTimer *reconnectTimer;
    DnsLookup * const dnsLookup;
    int connectingAttempts;
    FsmState fsmState = FsmState::Unknown;
    bool run;
    RcRunId runId;
    qint32 block_size;

    //--- old protocol variables
    RC_CLIENT_PROTO_ID protocolId = RC_CLIENT_PROTO_UNKNOWN;
    QByteArray recivedData;
    QTimer *dataTimer;
    bool dataTimerTimeout=false;
    quint64 curruptedSize=0;
    QStringList unknownCommands;
};

#endif // CLIENT_H
