/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef DEVICESTATUSDIALOG_H
#define DEVICESTATUSDIALOG_H

#include <QDialog>
#include <QTabWidget>

#include "base/DeviceIndex.h"
#include "ModularDeviceStatus.h"

class QTextEdit;

struct InfoViewHelper
{
    InfoViewHelper(QString title = {}, QWidget *parent = nullptr);
    void clear();
    void updateText(const QString &s);

    QString title;
    QString text;
    QTextEdit *textEdit = nullptr;
};

class DeviceStatusDialog : public QDialog
{
    Q_OBJECT
public:
    enum class InfoType{ Info, Sdb, Ports, Wr , FeLink };
    explicit DeviceStatusDialog(QWidget *parent = nullptr, Qt::WindowFlags f = Qt::WindowFlags());
    void setMultiStatus(const QMap<DeviceIndex, ModularDeviceStatus> &r);

    void setIndex(const DeviceIndex &r); // TODO: make private

public slots:
    void setStatus(const ModularDeviceStatus &r);
    void selectDevice(const DeviceIndex &r);
    void showDevice(const DeviceIndex &r);

protected:
    void resizeEvent(QResizeEvent *event) override;

private:
    void updateView();
    void setInitialSize();
    void adjustSize();

    DeviceIndex currentDeviceIndex;
    QMap<DeviceIndex, ModularDeviceStatus> statusMap;
    QMap<InfoType, InfoViewHelper> infoViewHelper;
    QTabWidget tabWidget;
    bool resized = false;
};

#endif // DEVICESTATUSDIALOG_H
