/*
**    Copyright 2021 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef REDISPUBLISHER_H
#define REDISPUBLISHER_H

#include <QObject>
#include <QTimer>
#include <QUrl>

#include "qredisclient/redisclient.h"
#include "../AbstractPublisher.h"

class RedisPublisher: public AbstractPublisher
{
    Q_OBJECT
public:
    RedisPublisher(const QString &serviceName, QObject *parent = nullptr);
    ~RedisPublisher();
    void pub(const QString &key, const QByteArray &data) override;
    bool event(QEvent* ev) override;

private slots:
    void connected();
    void disconnected();
    void error(const QString &);
    void log(const QString &);
    void deferredInit();

private:
    RedisClient::ConnectionConfig getConfig();
    void createConnection();
    void deleteConnection();
    void startReconnectTimer();

    const QString serviceName;
    RedisClient::Connection *connection = nullptr;
    RedisClient::ConnectionConfig config;
    bool enabled = true;
    QUrl url;
    QTimer * reconnectTimer = nullptr;
    int tryCount = 0;
};

#endif // REDISPUBLISHER_H
