/*
**    Copyright 2010-2022 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "MregDevice.h"

#include <QDebug>

#include "sdb/SdbReader.h"
#include "sdb/SdbTypes.h"

namespace mlink {

MregDevice::MregDevice(RegIOBase &p, int address, QObject *parent)
    : MlinkDevice(p, address, parent)
    , sdb(new sdb_t())
{}

MregDevice::~MregDevice()
{
    delete sdb;
}

sdb_t MregDevice::getSdb() const
{
    return *sdb;
}

bool MregDevice::readSdb(uint32_t address)
{
    int max_retries = 3;
    for (int i=0; i<max_retries; i++) {
        const auto &result = SdbReader::read(this, address);
        if (!result.has_value()) return false;
        const sdb_t &newSdb = result.value();
        if (!newSdb.isValid()) {
            qWarning().noquote() << getIdent() << QString("SDB read error (try %1 of %2)")
                                        .arg(i+1).arg(max_retries);
            continue;
        }
        *sdb = newSdb;
        return true;
    }
    return false;
}

bool MregDevice::detect()
{
    auto sdbFound = SdbReader::find(this);
    if (!sdbFound) {
        return {};
    }
    if (!readSdb(*sdbFound))
        return {};

    return true;
}

} // namespace mlink
