#include "ConfigListModel.h"

#include <QColor>

#include "json-tree-model/JsonTreeItem.h"

namespace {
const QColor GREEN_COLOR = QColor(95, 156, 95);
}

ConfigListModel::ConfigListModel(QObject *parent):
    JsonTreeModel(parent)
{
    set_key_name("Program index\nConfiguration name");
    set_value_name("Date of modification");
}

QVariant ConfigListModel::data(const QModelIndex &index, int role) const
{
    if (index.isValid()) {
        if(role == Qt::BackgroundRole) {
            if(parent(index).isValid()) {
                auto cin_var = data(index, Qt::UserRole);
                if(cin_var.isValid() &&
                        cin_var.value<ConfigIndexName>() == cur_cin) {
                        return GREEN_COLOR;
                }
            } else {
                auto *item = static_cast<JsonTreeItem*>(index.internalPointer());
                if(item->key() == cur_cin.getIndex().getIndex()) {
                    for(int r2=0; r2<rowCount(index); ++r2) {
                        auto cn_index = ConfigListModel::index(r2, JSON_MODEL_COL_KEY, index);
                        if(!cn_index.isValid())
                            continue;
                        auto *cn_item = static_cast<JsonTreeItem*>(cn_index.internalPointer());
                        if(cn_item->key() == cur_cin.getConfigurationName()) {
                            return GREEN_COLOR;
                        }
                    }
                }
            }
        }
        if(role == Qt::UserRole && parent(index).isValid()) {
            auto *item = static_cast<JsonTreeItem*>(index.internalPointer());
            if(item && item->parent()) {
                const auto &pi = ProgramIndex::fromString(item->parent()->key());
                ConfigIndexName cin(pi, ConfigurationName(item->key()));
                return QVariant::fromValue(cin);
            }
        }
    }

    return JsonTreeModel::data(index, role);
}

void ConfigListModel::set_current_cin(const ConfigIndexName &cin)
{
    auto prev_cin = cur_cin;
    cur_cin = cin;

    auto prev_index = getModelIndexByCin(prev_cin);
    auto cur_index = getModelIndexByCin(cur_cin);

    emit dataChanged(prev_index, prev_index, {Qt::BackgroundRole});
    emit dataChanged(cur_index, cur_index, {Qt::BackgroundRole});
}

QModelIndex ConfigListModel::getModelIndexByCin(const ConfigIndexName &cin) const
{
    for(int r=0; r<rowCount(); ++r) {
        auto pi_index = index(r, JSON_MODEL_COL_KEY);
        if(!pi_index.isValid())
            continue;
        auto *pi_item = static_cast<JsonTreeItem*>(pi_index.internalPointer());
        if(pi_item->key() != cin.getIndex().toString())
            continue;
        for(int r2=0; r2<rowCount(pi_index); ++r2) {
            auto cn_index = index(r2, JSON_MODEL_COL_KEY, pi_index);
            if(!cn_index.isValid())
                continue;
            JsonTreeItem *cn_item = static_cast<JsonTreeItem*>(cn_index.internalPointer());
            if(cn_item->key() == cin.getConfigurationName())
                return cn_index;
        }
        break;
    }
    return QModelIndex();
}
