#ifndef MongoDB_H
#define MongoDB_H

#include <optional>

#include "AbstractDB.h"
#include "mongoqt/MongoQt.h"
#include "mongoqt/MongoHost.h"

class MongoHost;
class RcRunId;
class QDateTime;

class MongoDB : public MongoQt, public AbstractDB
{
public:
    MongoDB() = default;
    virtual ~MongoDB() override = default;
    void connectToDb() override;
    std::optional<MongoHost> getConfiguredHost() const { return configuredHost; }
    void mongo_init();
    virtual ConfigDocument readConfigDocument(const ConfigSelector &cs) override;
    virtual bool writeConfigDocument(const ConfigDocument &cd) override;
    virtual bool writeConfigRun(const RcRunId &rc, const QJsonObject &jsonCfg) override;
    virtual int delete_config(const ConfigSelector &cs) override;
    virtual QMap<ProgramIndex, QMap<ConfigurationName, QDateTime> > get_config_map(const DaqModule &daq_module) override;
    QMap<ProgramIndex, QMap<ConfigurationName, QJsonObject> > get_json_map(
        const DaqModule &daq_module);
    [[deprecated("unused; toDel?")]]
    RcRunId check_run_number(const RcRunId &rc); //TODO: move to AbstractDB

private:
    void configureConnection();
    int remove_broken_docs(const QStringList &fields_to_check);
    int remove_duplicates();

    std::optional<MongoHost> configuredHost;
};

#endif // MongoDB_H
