/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef MSTREAMSTAT_H
#define MSTREAMSTAT_H

#include <QElapsedTimer>
#include "base/DeviceIndex.h"
#include "tcpserver/TcpServerStatus.h"

struct MStreamStat {
    MStreamStat() { clear(); }
    MStreamStat &operator-=(const MStreamStat &r)
    {
        packageCount -= r.packageCount;
        recvDataSize -= r.recvDataSize;
        retransmitedFragParts -= r.retransmitedFragParts;
        fragTotal -= r.fragTotal;
        fragClosed -= r.fragClosed;
        fragDroped -= r.fragDroped;
        fragMissed -= r.fragMissed;
        fragIncompled -= r.fragIncompled;
        goodFormatPackage -= r.goodFormatPackage;
        badFormatPackage -= r.badFormatPackage;
        missedEvents -= r.missedEvents;
        missedEventsRanges -= r.missedEventsRanges;
        incompleteEv -= r.incompleteEv;
        evCount -= r.evCount;
        jsonBlocks -= r.jsonBlocks;
        timeSliceBlocks -= r.timeSliceBlocks;
        acquiredAtMs -= r.acquiredAtMs;
        // other fields not subtractable
        return *this;
    }
    const MStreamStat operator-(const MStreamStat &rhs) const
    { return MStreamStat(*this) -= rhs; }

    void clear() {
        packageCount=0;
        recvDataSize=0;
        retransmitedFragParts=0;
        fragTotal=0;
        fragClosed=0;
        fragDroped=0;
        fragMissed=0;
        fragIncompled=0;
        goodFormatPackage=0;
        badFormatPackage=0;
        missedEvents=0;
        missedEventsRanges=0;
        incompleteEv=0;
        evCount=0;
        jsonBlocks=0;
        customBlocks=0;
        timeSliceBlocks=0;
        lastEvNum=0;
        upTime.start();
    }

    DeviceIndex index;
    // counters
    quint64 packageCount = 0;
    quint64 recvDataSize = 0;
    quint64 retransmitedFragParts = 0; // packages with lastClosed-Range<frId<lastClosed || dublicate start/end
    quint64 fragTotal = 0;
    quint64 fragClosed = 0;
    quint64 fragDroped = 0; // has incompleted size or it is a chFrame without trigFrame
    quint64 fragMissed = 0; // has incompleted size or it is a chFrame without trigFrame
    quint32 fragIncompled = 0; // incompleted fragments //#
    quint64 goodFormatPackage = 0;
    quint64 badFormatPackage = 0;
    quint64 missedEvents = 0;
    quint64 missedEventsRanges = 0;
    quint32 incompleteEv = 0;
    quint64 evCount = 0;
    quint64 jsonBlocks = 0;
    quint64 customBlocks = 0;
    quint64 timeSliceBlocks = 0;
    //
    quint64 lastEvNum = 0; // gauge
    QElapsedTimer upTime;
    quint64 acquiredAtMs = 0;
    TcpServerStatus tcpStatus;
};
Q_DECLARE_METATYPE(MStreamStat)

#endif // MSTREAMSTAT_H
