/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "PnpServerWithStats.h"

#include <QTimer>

static const int statisticsUpdatePeriodMs = 1000;

PnpServerWithStats::PnpServerWithStats(QObject *parent)
    : PNPServer(parent)
{
    QTimer::singleShot(statisticsUpdatePeriodMs, this, SLOT(sendStats()));
}

bool PnpServerWithStats::decode(const QByteArray &data, const QHostAddress &host)
{
    bool ret = PNPServer::decode(data, host);
    if (ret)
        pnpMessageStats.decodedMessageCount++;
    else
        pnpMessageStats.failedMessageCount++;
    return ret;
}

void PnpServerWithStats::sendStats()
{
    emit statsUpdated(pnpMessageStats);
    QTimer::singleShot(statisticsUpdatePeriodMs, this, SLOT(sendStats()));
}
