#include "RcProgramStateConverter.h"

#include "RcProgramState.h"
#include "base/ConfigConverterUtil.h"

namespace {
constexpr const char *k_index = "index";
constexpr const char *k_fsm = "fsm";
constexpr const char *k_clients = "clients";
constexpr const char *k_devices = "devices";
}


template<>
bool AbstractConfigConverter<RcDeviceState>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<RcDeviceState>::toJsonObject(const RcDeviceState &in)
{
    QVariantMap m;
    m[k_index] = QVariant::fromValue(in.index);
    m[k_fsm] = QVariant::fromValue(fsmStateToString(in.state));
    return fromVariantMap(m);
}

template<>
RcDeviceState AbstractConfigConverter<RcDeviceState>::fromJsonObject(const QJsonObject &m)
{
    RcDeviceState out;
    update_value(m, k_index, &out.index);
    QString state;
    update_value(m, k_fsm, &state);
    out.state = getFsmState(state);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<RcDeviceState>::toJsonValue(const RcDeviceState &in)
{
    return toJsonObject(in);
}

template<>
RcDeviceState AbstractConfigConverter<RcDeviceState>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<RcDeviceState>::registerSubTypes()
{
    AbstractConfigConverter<DeviceIndex>::registerTypes();
}


// ---


template<>
bool AbstractConfigConverter<RcProgramState>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<RcProgramState>::toJsonObject(const RcProgramState &in)
{
    QVariantMap m;
    m[k_index] = QVariant::fromValue(in.index);
    m[k_fsm] = QVariant::fromValue(fsmStateToString(in.fsm));
    if(!in.clients.isEmpty())
        m[k_clients] = ConfigConverterUtil::toJsonValue(in.clients);
    if(!in.devices.isEmpty())
        m[k_devices] = ConfigConverterUtil::toJsonValue(in.devices);
    return fromVariantMap(m);
}

template<>
RcProgramState AbstractConfigConverter<RcProgramState>::fromJsonObject(const QJsonObject &m)
{
    RcProgramState out;
    update_value(m, k_index, &out.index);
    QString state;
    update_value(m, k_fsm, &state);
    out.fsm = getFsmState(state);
    out.clients = ConfigConverterUtil::toMap<ClientIndex, RcProgramState>(m[k_clients]);
    out.devices = ConfigConverterUtil::toMap<DeviceIndex, RcDeviceState>(m[k_devices]);

    return out;
}

template<>
QJsonValue AbstractConfigConverter<RcProgramState>::toJsonValue(const RcProgramState &in)
{
    return toJsonObject(in);
}

template<>
RcProgramState AbstractConfigConverter<RcProgramState>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<RcProgramState>::registerSubTypes()
{
    AbstractConfigConverter<RcDeviceState>::registerTypes();
    AbstractConfigConverter<ClientIndex>::registerTypes();
    AbstractConfigConverter<DeviceIndex>::registerTypes();
}

// ---
